@@ -228,7 +228,8 @@ static int softReset(int host)
 	outb(reg[host].ctr,0x2);	/* ꥻåȲ|߶ػ */
 /*	mili_timer(20);*/				/* 20ms wait */
 	mili_timer(5);				/* 5ms wait */
-	if((checkBusy(reg[host].astr)&BSY_BIT)!=0)return PRINT_ERR(EDBUSY,"softReset");
+	if((checkBusy(reg[host].astr)&BSY_BIT)!=0)
+		return -EDBUSY;
 
 	return 0;
 }
@@ -241,9 +242,10 @@ static int softReset(int host)
  */
 static void setIntr(int host,int flag)
 {
-	if(flag==currentIntr[host])return;
+	if (flag == currentIntr[host])
+		return;
 
-	if(flag==INTR_DISABLE)
+	if (flag == INTR_DISABLE)
 	{
 		set_irq_mask(irqNum[host]);
 		outb(reg[host].ctr,0x2);			/* Disable interrupt */
@@ -316,18 +318,22 @@ static int deviceSelect(int host,int dev,int lba24_27)
 
 	if(((error=checkBusy(reg[host].astr))&(DRQ_BIT|BSY_BIT))!=0)
 	{
-		if(error&DRQ_BIT)return PRINT_ERR(EDERRE,"deviceSelect");
-		if(error&BSY_BIT)return PRINT_ERR(EDBUSY,"deviceSelect");
+		if(error&DRQ_BIT)
+			return -EDERRE;
+		if(error&BSY_BIT)
+			return -EDBUSY;
 	}
 
 	/* Device select */
 	outb(reg[host].dhr,(uchar)(LBA_BIT|(dev<<4)|lba24_27));
-	micro_timer(1);					/* 400ns wait */
+	micro_timer(1);						/* 400ns wait */
 
 	if(((error=checkBusy(reg[host].astr))&(DRQ_BIT|BSY_BIT))!=0)
 	{
-		if(error&DRQ_BIT)return PRINT_ERR(EDERRE,"deviceSelect");
-		if(error&BSY_BIT)return PRINT_ERR(EDBUSY,"deviceSelect");
+		if(error&DRQ_BIT)
+			return -EDERRE;
+		if(error&BSY_BIT)
+			return -EDBUSY;
 	}
 	return 0;
 }
@@ -343,6 +349,10 @@ static int readPio(int host,int dev,void *buf,int block,int count)
 {
 	int error;
 	int i,j,last;
+	union{
+		void *pv;
+		uint ui;
+	}value;
 
 
 	last=block/2;
@@ -350,7 +360,9 @@ static int readPio(int host,int dev,void *buf,int block,int count)
 	{
 		if(((error=checkBusy(reg[host].str))&(BSY_BIT|DRQ_BIT))!=DRQ_BIT)return error;
 		for(j=0;j<last;++j)((short*)buf)[j]=inw(reg[host].dtr);
-		(uint)buf+=block;
+		value.pv = buf;
+		value.ui += block;
+		buf = value.pv;
 	}
 
 	return checkBusy(reg[host].str);
@@ -367,6 +379,10 @@ static int writePio(int host,int dev,void *buf,int block,int count)
 {
 	int error;
 	int i,j,last;
+	union{
+		void *pv;
+		uint ui;
+	}value;
 
 
 	last=block/2;
@@ -374,7 +390,9 @@ static int writePio(int host,int dev,void *buf,int block,int count)
 	{
 		if(((error=checkBusy(reg[host].str))&(BSY_BIT|DRQ_BIT))!=DRQ_BIT)return error;
 		for(j=0;j<last;++j)outw(reg[host].dtr,((short*)buf)[j]);
-		(uint)buf+=block;
+		value.pv = buf;
+		value.ui += block;
+		buf = value.pv;
 	}
 
 	return checkBusy(reg[host].str);
@@ -390,9 +408,9 @@ static int writePio(int host,int dev,void *buf,int block,int count)
 static int readDma(int host,int dev,void *buf,int sector_size,int count)
 {
 	/* Set PRD */
-	prd[host].physAddr=buf;
-	prd[host].count=sector_size*count|PRD_EOT;
-	outdw(ide_base[host]+IDE_BMIDTP,(uint)&prd[host]);
+	prd[host].physAddr = buf;
+	prd[host].count = sector_size * count | PRD_EOT;
+	outdw(ide_base[host] + IDE_BMIDTP,(uint)&prd[host]);
 
 	/* Mask interrupt. */
 /*	set_irq_mask(irqNum[host]);*/
@@ -401,10 +419,10 @@ static int readDma(int host,int dev,void *buf,int sector_size,int count)
 	 * Хޥơ쥸γߥե饰򥯥ꥢʤ
 	 * ߤȯʤ褦
 	 */
-	outb(ide_base[host]+IDE_BMIS,0x6);			/* Clear interrupt bit and error bit */
-	outb(ide_base[host]+IDE_BMIC,0x9);			/* Start read Bus Master */
+	outb(ide_base[host] + IDE_BMIS,0x6);			/* Clear interrupt bit and error bit */
+	outb(ide_base[host] + IDE_BMIC,0x9);			/* Start read Bus Master */
 	wait_intr(&wait_intr_queue[host],IDE_BM_TIMEOUT,TASK_DEVICE_WAIT);		/* Wait interrupt */
-	outb(ide_base[host]+IDE_BMIC,0);			/* Stop Bus Master */
+	outb(ide_base[host] + IDE_BMIC,0);				/* Stop Bus Master */
 
 	return inb(reg[host].str);
 }
@@ -447,10 +465,13 @@ static int transferAta(int host,int dev,int trans_mode,void *buf,int count,uint
 	int error;
 
 
-	if(conectDev[host][dev].mode==PIO)setIntr(host,INTR_DISABLE);
-	else setIntr(host,INTR_ENABLE);
+	if (conectDev[host][dev].mode == PIO)
+		setIntr(host,INTR_DISABLE);
+	else
+		setIntr(host,INTR_ENABLE);
 
-	if((error=deviceSelect(host,dev,begin>>24))!=0)return error;
+	if ((error = deviceSelect(host,dev,begin>>24)) != 0)
+		return error;
 
 	outb(reg[host].scr,(uchar)count);
 	outb(reg[host].snr,(uchar)begin);
@@ -458,38 +479,40 @@ static int transferAta(int host,int dev,int trans_mode,void *buf,int count,uint
 	outb(reg[host].chr,(uchar)(begin>>16));
 
 	/* PIO transfer */
-	if(conectDev[host][dev].mode==PIO)
+	if (conectDev[host][dev].mode == PIO)
 	{
-		if(trans_mode==READ)
+		if (trans_mode == READ)
 		{
 			outb(reg[host].cmr,0x20);
-			error=readPio(host,dev,buf,ATA_SECTOR_SIZE,count);
+			error = readPio(host,dev,buf,ATA_SECTOR_SIZE,count);
 		}
 		else
 		{
 			outb(reg[host].cmr,0x30);
-			error=writePio(host,dev,buf,ATA_SECTOR_SIZE,count);
+			error = writePio(host,dev,buf,ATA_SECTOR_SIZE,count);
 		}
 	}
 	/* DMA transfer */
 	else
 	{
-		if(trans_mode==READ)
+		if (trans_mode == READ)
 		{
 			outb(reg[host].cmr,0xc8);
-			error=readDma(host,dev,buf,ATA_SECTOR_SIZE,count);
+			error = readDma(host,dev,buf,ATA_SECTOR_SIZE,count);
 		}
 		else
 		{
 			outb(reg[host].cmr,0xca);
-			error=writeDma(host,dev,buf,ATA_SECTOR_SIZE,count);
+			error = writeDma(host,dev,buf,ATA_SECTOR_SIZE,count);
 		}
 	}
 
-	if((error&(BSY_BIT|DRQ_BIT|ERR_BIT))!=0)
+	if ((error & (BSY_BIT | DRQ_BIT | ERR_BIT)) != 0)
 	{
-		if(error&(DRQ_BIT|ERR_BIT))return PRINT_ERR(EDERRE,"transferAta");
-		if(error&BSY_BIT)return PRINT_ERR(EDBUSY,"transferAta");
+		if (error & (DRQ_BIT | ERR_BIT))
+			return -EDERRE;
+		if (error & BSY_BIT)
+			return -EDBUSY;
 	}
 
 	return 0;
@@ -508,24 +531,24 @@ static int initIdeBusmaster(int host,PCI_INFO *ide)
 
 
 	/* Serch IDE */
-	ide->bus=ide->dev=ide->func=0;
-	if(search_pci_class(PCI_CLASS_IDE_BSMST,ide)<0)
-		return PRINT_ERR(ENODEV,"initIdeBusmaster");
+	ide->bus = ide->dev = ide->func = 0;
+	if (search_pci_class(PCI_CLASS_IDE_BSMST,ide) < 0)
+		return -ENODEV;
 
 	/* Test Bus Master enable bit on */
-	com=read_pci_config(ide,PCI_CONF_COMMAND);
-	writew_pci_config(ide,PCI_CONF_COMMAND,com|PCI_COMMAND_BSMST);
-	if((read_pci_config(ide,PCI_CONF_COMMAND)&PCI_COMMAND_BSMST)==0)
-		return PRINT_ERR(EDNOFUNC,"initIdeBusmaster");
+	com = read_pci_config(ide,PCI_CONF_COMMAND);
+	writew_pci_config(ide,PCI_CONF_COMMAND,com | PCI_COMMAND_BSMST);
+	if ((read_pci_config(ide,PCI_CONF_COMMAND) & PCI_COMMAND_BSMST) == 0)
+		return -EDNOFUNC;
 
 	/* Get IO base address */
-	if((ide_base[0]=read_pci_config(ide,PCI_CONF_BMBASE))==0)
-		return PRINT_ERR(EDNOFUNC,"initIdeBusmaster");
-	ide_base[0]&=0xfff0;
-	ide_base[1]=ide_base[0]+IDE_BMIO_SECOND;
+	if ((ide_base[0] = read_pci_config(ide,PCI_CONF_BMBASE)) == 0)
+		return -EDNOFUNC;
+	ide_base[0] &= 0xfff0;
+	ide_base[1] = ide_base[0] + IDE_BMIO_SECOND;
 
 	/* Reset Bus Master */
-	outb(ide_base[host]+IDE_BMIC,0);
+	outb(ide_base[host] + IDE_BMIC,0);
 
 	return 0;
 }
@@ -571,8 +594,10 @@ static int resetDevice(int host,int dev)
 
 	if(((error=checkBusy(reg[host].astr))&(BSY_BIT|ERR_BIT))!=0)
 	{
-		if(error&ERR_BIT)return PRINT_ERR(EDERRE,"resetDevice");
-		if(error&BSY_BIT)return PRINT_ERR(EDBUSY,"resetDevice");
+		if(error&ERR_BIT)
+			return -EDERRE;
+		if(error&BSY_BIT)
+			return -EDBUSY;
 	}
 	return 0;
 }
@@ -599,8 +624,10 @@ static int identifyDevice(int host,int dev,int drv,void *buf)
 	/* Read data */
 	if(((error=readPio(host,dev,buf,IDENTIFY_SIZE,1))&(BSY_BIT|DRQ_BIT|ERR_BIT))!=0)
 	{
-		if(error&(DRQ_BIT|ERR_BIT))return PRINT_ERR(EDERRE,"identifyDevice");
-		if(error&BSY_BIT)return PRINT_ERR(EDBUSY,"identifyDevice");
+		if(error&(DRQ_BIT|ERR_BIT))
+			return -EDERRE;
+		if(error&BSY_BIT)
+			return -EDBUSY;
 	}
 	return 0;
 }
@@ -626,8 +653,10 @@ static int idleImmediateDevice(int host,int dev)
 
 	if(((error=checkBusy(reg[host].astr))&(BSY_BIT|ERR_BIT))!=0)
 	{
-		if(error&ERR_BIT)return PRINT_ERR(EDERRE,"idleImmediateDevice");
-		if(error&BSY_BIT)return PRINT_ERR(EDBUSY,"idleImmediateDevice");
+		if(error&ERR_BIT)
+			return -EDERRE;
+		if(error&BSY_BIT)
+			return -EDBUSY;
 	}
 	return 0;
 }
@@ -656,8 +685,10 @@ static int initDeviceParam(int host,int dev,uchar head,uchar sectors)
 
 	if(((error=checkBusy(reg[host].astr))&(BSY_BIT|ERR_BIT))!=0)
 	{
-		if(error&ERR_BIT)return PRINT_ERR(EDERRE,"initDeviceParam");
-		if(error&BSY_BIT)return PRINT_ERR(EDBUSY,"initDeviceParam");
+		if(error&ERR_BIT)
+			return -EDERRE;
+		if(error&BSY_BIT)
+			return -EDBUSY;
 	}
 	return 0;
 }
@@ -685,8 +716,10 @@ static int setFeatures(int host,int dev,uchar subcommand,uchar trans_mode)
 
 	if(((error=checkBusy(reg[host].astr))&(BSY_BIT|ERR_BIT))!=0)
 	{
-		if(error&ERR_BIT)return PRINT_ERR(EDERRE,"setFeatures");
-		if(error&BSY_BIT)return PRINT_ERR(EDBUSY,"setFeatures");
+		if(error&ERR_BIT)
+			return -EDERRE;
+		if(error&BSY_BIT)
+			return -EDBUSY;
 	}
 	return 0;
 }
@@ -754,8 +787,10 @@ static int issuePacketCommand(int host,int dev,PACKET_PARAM *param)
 	outb(reg[host].clr,0xff);
 	outb(reg[host].chr,0xff);
 	outb(reg[host].cmr,0xa0);
-	if((checkBusy(reg[host].str)&(DRQ_BIT|CHK_BIT))!=DRQ_BIT)return PRINT_ERR(EDERRE,"issuePacketCommand");
-	if((inb(reg[host].irr)&(CD_BIT|IO_BIT))!=CD_BIT)return PRINT_ERR(EDERRE,"issuePacketCommand");
+	if((checkBusy(reg[host].str)&(DRQ_BIT|CHK_BIT))!=DRQ_BIT)
+		return -EDERRE;
+	if((inb(reg[host].irr)&(CD_BIT|IO_BIT))!=CD_BIT)
+		return -EDERRE;
 
 	/* DMA transfer */
 	if(param->feutures&PACK_DMA)
@@ -771,13 +806,16 @@ static int issuePacketCommand(int host,int dev,PACKET_PARAM *param)
 			wait_intr(&wait_intr_queue[host],2000,TASK_DEVICE_WAIT);			/* Wait interrupt */
 
 			error=(int)inb(reg[host].str);
-			if(error&ERR_BIT)return PRINT_ERR(EDERRE,"issuePacketCommand");		/* Packet command Error */
-			if((error&DRQ_BIT)==0)return 0;				/* Non data transfer */
+			if(error&ERR_BIT)
+				return -EDERRE;		/* Packet command Error */
+			if((error&DRQ_BIT)==0)
+				return 0;				/* Non data transfer */
 
 			if((error=deviceSelect(host,dev,0))!=0)return error;
 			wait_intr(&wait_intr_queue[host],2000,TASK_DEVICE_WAIT);		/* Wait interrupt */
 			outb(reg[host].cmr,0xa2);					/* Issue service command */
-			if((checkBusy(reg[host].str)&(BSY_BIT|DRQ_BIT))!=DRQ_BIT)return PRINT_ERR(EDBUSY,"issuePacketCommand");
+			if((checkBusy(reg[host].str)&(BSY_BIT|DRQ_BIT))!=DRQ_BIT)
+				return -EDBUSY;
 		}
 
 		/* Data transfer */
@@ -798,8 +836,10 @@ static int issuePacketCommand(int host,int dev,PACKET_PARAM *param)
 		for(i=0;i<6;++i)outw(dtr,((short*)param->packet)[i]);
 
 		error=checkBusy(reg[host].str);
-		if(error&ERR_BIT)return PRINT_ERR(EDERRE,"issuePacketCommand");		/* Packet command Error */
-		if((error&DRQ_BIT)==0)return 0;			/* Non data transfer */
+		if(error&ERR_BIT)
+			return -EDERRE;		/* Packet command Error */
+		if((error&DRQ_BIT)==0)
+			return 0;			/* Non data transfer */
 
 		/* Overrapped */
 		if(param->feutures&PACK_OVL)
@@ -807,7 +847,8 @@ static int issuePacketCommand(int host,int dev,PACKET_PARAM *param)
 			if((error=deviceSelect(host,dev,0))!=0)return error;
 			wait_intr(&wait_intr_queue[host],2000,TASK_DEVICE_WAIT);			/* Wait interrupt */
 			outb(reg[host].cmr,0xa2);						/* Issue service command */
-			if((checkBusy(reg[host].str)&(BSY_BIT|DRQ_BIT))!=DRQ_BIT)return PRINT_ERR(EDBUSY,"issuePacketCommand");
+			if((checkBusy(reg[host].str)&(BSY_BIT|DRQ_BIT))!=DRQ_BIT)
+				return -EDBUSY;
 		}
 
 		/* Data transfer */
@@ -822,8 +863,10 @@ static int issuePacketCommand(int host,int dev,PACKET_PARAM *param)
 	/* Last check */
 	if((error&(BSY_BIT|DRQ_BIT|ERR_BIT))!=0)
 	{
-		if(error&(DRQ_BIT|ERR_BIT))return PRINT_ERR(EDERRE,"issuePacketCommand");
-		if(error&BSY_BIT)return PRINT_ERR(EDBUSY,"issuePacketCommand");
+		if(error&(DRQ_BIT|ERR_BIT))
+			return -EDERRE;
+		if(error&BSY_BIT)
+			return -EDBUSY;
 	}
 	return 0;
 }
@@ -876,8 +919,8 @@ static int requestSense(int host,int dev)
 		case 0x62800:return 0;			/* Not ready to ready change */
 		case 0x62900:					/* Power on,Reset */
 		case 0x20401:return 1;			/* Becoming ready */
-		case 0x63a00:return PRINT_ERR(ENOMEDIUM,"requestSense");	/* Medium not present */
-		default:return PRINT_ERR(EDERRE,"requestSense");
+		case 0x63a00:return -ENOMEDIUM;	/* Medium not present */
+		default:     return -EDERRE;
 	}
 }
 
@@ -994,7 +1037,8 @@ static int changeMode(int host,int dev,int mode)
 
 	if(conectDev[host][dev].mode==mode)return 0;
 
-	if((id_info=(ID_INFO*)kmalloc(IDENTIFY_SIZE))==NULL)return PRINT_ERR(EDNOMEM,"changeMode");
+	if((id_info=(ID_INFO*)kmalloc(IDENTIFY_SIZE))==NULL)
+		return -EDNOMEM;
 
 	/* Read identify infomation */
 	if((error=identifyDevice(host,dev,conectDev[host][dev].type,id_info))!=0)return error;
@@ -1011,7 +1055,8 @@ static int changeMode(int host,int dev,int mode)
 		if(id_info->multi_dma&M_DMA2)subcm=SUB_M_DMA|2;
 		else if(id_info->multi_dma&M_DMA1)subcm=SUB_M_DMA|1;
 		else if(id_info->multi_dma&M_DMA0)subcm=SUB_M_DMA|0;
-		else return PRINT_ERR(EDNOFUNC,"changeMode");
+		else 
+			return -EDNOFUNC;
 
 		/*
 		 * ULTRA DMAбΥɥ饤֤ˤĤƤϡBIOSIDEULTRA DMA
@@ -1074,7 +1119,7 @@ static int changeMode(int host,int dev,int mode)
 	else if(mode==U_DMA)
 	{
 		if((id_info->ultra_dma&(U_DMA5|U_DMA4|U_DMA3|U_DMA2|U_DMA1|U_DMA0))==0)
-			return PRINT_ERR(EDNOFUNC,"changeMode");
+			return -EDNOFUNC;
 
 		/*
 		 * ULTRA DMAбΥɥ饤֤ˤĤƤϡBIOSIDEULTRA DMA
@@ -1254,10 +1299,11 @@ VIA:   			value=read_pci_config(&ide,0x50);
     				break;
 				}
 			default:
-				return PRINT_ERR(EDNOFUNC,"changeMode");
+				return -EDNOFUNC;
 		}
 	}
-	else return PRINT_ERR(EDINVAL,"changeMode");
+	else 
+		return -EDINVAL;
 
 	conectDev[host][dev].mode=mode;
 
@@ -1326,21 +1372,24 @@ static int testAtapi(int,int);
  */
 static inline int transfer(int host,int dev,int mode,void *buf,size_t blocks,size_t begin)
 {
-	int error;
 	int rest;
 
 
-	if(blocks==0)return 0;
-	if(begin+blocks>conectDev[host][dev].allSectors)return PRINT_ERR(EDINVAL,"transfer");
+	if (blocks == 0)
+		return 0;
+	if (begin + blocks > conectDev[host][dev].allSectors)
+		return -EDINVAL;
 
 	waitSemaphore(&wait_queue[host]);
 	{
 		/* SMPʤߤƱcpuȯ褦ˤ */
-		if(MFPS_addres)set_intr_cpu(irqNum[host],get_current_cpu());
+		if(MFPS_addres)
+			set_intr_cpu(irqNum[host],get_current_cpu());
 
 		/* ž */
-		if((error=conectDev[host][dev].transfer(host,dev,mode,buf,blocks,begin))!=0)rest=error;
-		else rest=blocks;
+		rest = conectDev[host][dev].transfer(host,dev,mode,buf,blocks,begin);
+		if (rest == 0)
+			rest = blocks;
 	}
 	wakeSemaphore(&wait_queue[host]);
 
@@ -1356,9 +1405,9 @@ static int openHda()
 {
 	switch(conectDev[0][0].type)
 	{
-		case ATA:return 0;
+		case ATA:  return 0;
 		case ATAPI:return testAtapi(0,0);
-		default:return PRINT_ERR(EDNODEV,"openHda");
+		default:   return -EDNODEV;
 	}
 }
 
@@ -1366,9 +1415,9 @@ static int openHdb()
 {
 	switch(conectDev[0][1].type)
 	{
-		case ATA:return 0;
+		case ATA:  return 0;
 		case ATAPI:return testAtapi(0,1);
-		default:return PRINT_ERR(EDNODEV,"openHdb");
+		default:   return -EDNODEV;
 	}
 }
 
@@ -1376,9 +1425,9 @@ static int openHdc()
 {
 	switch(conectDev[1][0].type)
 	{
-		case ATA:return 0;
+		case ATA:  return 0;
 		case ATAPI:return testAtapi(1,0);
-		default:return PRINT_ERR(EDNODEV,"openHdc");
+		default:   return -EDNODEV;
 	}
 }
 
@@ -1386,9 +1435,9 @@ static int openHdd()
 {
 	switch(conectDev[1][1].type)
 	{
-		case ATA:return 0;
+		case ATA:  return 0;
 		case ATAPI:return testAtapi(1,1);
-		default:return PRINT_ERR(EDNODEV,"openHdd");
+		default:   return -EDNODEV;
 	}
 }
 
@@ -1432,32 +1481,37 @@ static int writeHdd(void *buf,size_t size,size_t begin)
 	return transfer(1,1,WRITE,buf,size,begin);
 }
 
-static int ioctlHda(int prm_num,int cmd,...)
+static int ioctlHda(int cmd,...)
 {
 	return 0;
 }
 
-static int ioctlHdb(int prm_num,int cmd,...)
+static int ioctlHdb(int cmd,...)
 {
 	return 0;
 }
 
-static int ioctlHdc(int prm_num,int cmd,...)
+static int ioctlHdc(int cmd,...)
 {
 	return 0;
 }
 
-static int ioctlHdd(int prm_num,int cmd,...)
+static int ioctlHdd(int cmd,...)
 {
 	return 0;
 }
 
-/* GLOBAL */
+static int select(int flag,int polling)
+{
+	return 1;
+}
+
+
 static DEV_INFO hd_info[2][2]={
-	{{"hda",0,0,0,openHda,readHda,writeHda,ioctlHda},
-	 {"hdb",0,0,0,openHdb,readHdb,writeHdb,ioctlHdb}},
-	{{"hdc",0,0,0,openHdc,readHdc,writeHdc,ioctlHdc},
-	 {"hdd",0,0,0,openHdd,readHdd,writeHdd,ioctlHdd}}
+	{{"hda",0,0,0,openHda,readHda,writeHda,ioctlHda,select},
+	 {"hdb",0,0,0,openHdb,readHdb,writeHdb,ioctlHdb,select}},
+	{{"hdc",0,0,0,openHdc,readHdc,writeHdc,ioctlHdc,select},
+	 {"hdd",0,0,0,openHdd,readHdd,writeHdd,ioctlHdd,select}}
 };
 
 
@@ -1494,6 +1548,7 @@ static int testAtapi(int host,int dev)
  *
  ************************************************************************************************/
 
+
 /*
  * PRIVATE
  * ɤΥӥåǥȥ륨ǥѤ
@@ -1546,7 +1601,8 @@ int initAta()
 	/* ATA common 饹ν */
 	initCommon();
 
-	if((id_info=(ID_INFO*)kmalloc(IDENTIFY_SIZE))==NULL)return PRINT_ERR(EDNOMEM,"initAta");
+	if((id_info=(ID_INFO*)kmalloc(IDENTIFY_SIZE))==NULL)
+		return -EDNOMEM;
 
 	/* ³ǥХȽꤹ */
 	for(i=0;i<2;++i)
@@ -1662,6 +1718,7 @@ int initAta()
 
 	return 0;
 }
+
 /******************************************************************/
 void test_hd()
 {
