@@ -6,17 +6,18 @@
  */
 
 
-#include"types.h"
-#include"lib.h"
-#include"lock.h"
-#include"device.h"
-#include"proc.h"
-#include"interrupt.h"
-#include"signal.h"
-#include"keymap.h"
-#include"term.h"
-#include"errno.h"
-#include"console.h"
+#include<types.h>
+#include<lib.h>
+#include<lock.h>
+#include<device.h>
+#include<wait.h>
+#include<interrupt.h>
+#include<signal.h>
+#include<keymap.h>
+#include<term.h>
+#include<errno.h>
+#include<mm.h>
+#include<console.h>
 
 
 /***************************************************************************************
@@ -35,12 +36,12 @@ TERM_CTL termCtl;
  * ҥץƱGIDΥץ˥ʥ롣
  * parameters : begin process,groupe ID,signal
  */
-static void sendSignalToGrp(PROC *parent,gid_t gid,int signal)
+static void sendSignalToGrp(PROC *parent,pid_t pgid,int signal)
 {
 	PROC *p;
 
 
-	for(p=parent;(p=getNextProc(p,gid))!=NULL;)
+	for(p=parent;(p=getNextProc(p,pgid))!=NULL;)
 		sendSignal(p,signal);
 }
 
@@ -64,15 +65,15 @@ static struct termios termio={
 static inline void initTerm()
 {
 	/* termiosꡣ */
-	termio.c_cc[VINTR]=	'c'&0x1f;
-	termio.c_cc[VQUIT]=	'/'&0x1f;
-	termio.c_cc[VSTART]='q'&0x1f;
-	termio.c_cc[VSTOP]=	's'&0x1f;
-	termio.c_cc[VSUSP]=	'z'&0x1f;
-	termio.c_cc[VEOF]=	'd'&0x1f;
-	termio.c_cc[VEOL]=	0;
-	termio.c_cc[VERASE]='h'&0x1f;
-	termio.c_cc[VKILL]=	'u'&0x1f;
+	termio.c_cc[VINTR]=	CINTR;
+	termio.c_cc[VQUIT]=	CQUIT;
+	termio.c_cc[VSTART]=CSTART;
+	termio.c_cc[VSTOP]=	CSTOP;
+	termio.c_cc[VSUSP]=	CDSUSP;
+	termio.c_cc[VEOF]=	CEOF;
+	termio.c_cc[VEOL]=	CEOL;
+	termio.c_cc[VERASE]=CERASE;
+	termio.c_cc[VKILL]=	CKILL;
 }
 
 
@@ -82,7 +83,7 @@ static inline void initTerm()
  */
 static inline void isForeground()
 {
-	if(get_current_task()->gid!=termCtl.foregrnd)
+	if(get_current_task()->pgid!=termCtl.foregrnd)
 		del_from_schedule(TASK_SIGNAL_WAIT);
 }
 
@@ -111,7 +112,7 @@ void releaseTerm(PROC *proc)
 
 enum{
 	/* VGA Video ram. */
-	VRAM_BASE_ADDR=0xb8000,		/* Start address of video memory */
+	VRAM_BASE_ADDR=0xb8000,	/* Start address of video memory */
 	VRAM_SIZE=0x8000,		/* Size of video memory */
 
 	/* CRT IO registers. */
@@ -126,19 +127,20 @@ enum{
 	SCR_COLUMNS=80,			/* Number of columns. */
 	SCR_LAST=				/* Video ram last address. */
 		VRAM_SIZE/2-SCR_LINES*SCR_COLUMNS-SCR_COLUMNS*3,
+
+	TAB=8,					/* Tab characters. */
 };
 
 
 typedef struct{
 	int viewTop;	/* ӥǥ꡼β̤γϰ֡ */
 	int start;		/* ϰ֡ */
-	int cursor;		/* ֡ */
 }SCREEN;
 
 
 /* PRIVATE */
 static char *vram=(char*)VRAM_BASE_ADDR;
-static SCREEN screen={0,0,0};
+static SCREEN screen;
 
 
 /*
@@ -148,9 +150,9 @@ static SCREEN screen={0,0,0};
 static inline void updateCursor()
 {
 	outb(CTL_ADDR,CURSOR_ADDR_HIGH);
-	outb(CTL_DATA,screen.cursor>>8);
+	outb(CTL_DATA,screen.start>>8);
 	outb(CTL_ADDR,CURSOR_ADDR_LOW);
-	outb(CTL_DATA,screen.cursor);
+	outb(CTL_DATA,screen.start);
 }
 
 
@@ -181,19 +183,19 @@ static void scrollup()
 	int i,last;
 
 
-	screen.viewTop=ROUNDUP(screen.cursor-SCR_COLUMNS*SCR_LINES+1,SCR_COLUMNS);
+	screen.viewTop=ROUNDUP(screen.start-SCR_COLUMNS*SCR_LINES+1,SCR_COLUMNS);
 
 	if(screen.viewTop>=SCR_LAST)
 	{
 		/* Video ramƬ˰ư */
 		p=&vram[screen.viewTop*2];
-		last=(screen.cursor-screen.viewTop)*2;
+		last=(screen.start-screen.viewTop)*2;
 		for(i=0;i<last;i+=2)vram[i]=p[i];
 
 		/* Video ram0ꥢ */
 		for(;i<(SCR_LAST+SCR_COLUMNS*SCR_LINES)*2;i+=2)vram[i]=0;
 
-		screen.start=screen.cursor-=screen.viewTop;
+		screen.start-=screen.viewTop;
 		screen.viewTop=0;
 	}
 
@@ -238,15 +240,15 @@ static inline void putNumIntoScreen(char *buf,int size)
 /* Left cursor. */
 static inline void leftCursor(int n)
 {
-	screen.start=screen.cursor-=n;
+	screen.start-=n;
 	updateCursor();
 }
 
 /* Right cursor. */
 static inline void rightCursor(int n)
 {
-	screen.start=screen.cursor+=n;
-	if(screen.cursor-screen.viewTop>=SCR_COLUMNS*SCR_LINES)
+	screen.start+=n;
+	if(screen.start-screen.viewTop>=SCR_COLUMNS*SCR_LINES)
 		scrollup();
 	updateCursor();
 }
@@ -254,15 +256,24 @@ static inline void rightCursor(int n)
 /* Carriage retun. */
 static inline void carriageRetun()
 {
-	screen.start=screen.cursor=ROUNDDOWN(screen.cursor,SCR_COLUMNS);
+	screen.start=ROUNDDOWN(screen.start,SCR_COLUMNS);
 	updateCursor();
 }
 
 /* New line. */
 static inline void newLine()
 {
-	screen.start=screen.cursor=ROUNDUP(screen.cursor+1,SCR_COLUMNS);
-	if(screen.cursor-screen.viewTop>=SCR_COLUMNS*SCR_LINES)
+	screen.start=ROUNDUP(screen.start+1,SCR_COLUMNS);
+	if(screen.start-screen.viewTop>=SCR_COLUMNS*SCR_LINES)
+		scrollup();
+	updateCursor();
+}
+
+/* Tab. */
+static inline void tab()
+{
+	screen.start=ROUNDUP(screen.start+1,TAB);
+	if(screen.start-screen.viewTop>=SCR_COLUMNS*SCR_LINES)
 		scrollup();
 	updateCursor();
 }
@@ -294,8 +305,9 @@ static void initScreen()
  *
  ***************************************************************************************/
 
- enum{
+enum{
 	CONSOLE_BUF_SIZE=90,	/* Line buffer size. */
+	CONSOLE_EOF=0xff,		/* EOF character. */
 };
 
 
@@ -385,11 +397,10 @@ static inline void leftCursorBuf()
 /* Right cursor. */
 static inline void rightCursorBuf()
 {
-	if(lbuf.crt<CONSOLE_BUF_SIZE)
+	if(lbuf.crt<lbuf.last)
 	{
 		rightCursor(1);
 		++lbuf.crt;
-		if(lbuf.last<lbuf.crt)lbuf.last=lbuf.crt;
 	}
 }
 
@@ -411,12 +422,28 @@ static inline void newLineBuf()
 	lbuf.lineTop=lbuf.crt=lbuf.last;
 }
 
+/*
+ * EOF.
+ * return : ʸʤ=1 or 0
+ */
+static int Eof()
+{
+	if(lbuf.lineTop==lbuf.last)	/* ʸʤ */
+	{
+		lbuf.buf[lbuf.last++]=CONSOLE_EOF;
+		lbuf.lineTop=lbuf.crt=lbuf.last;
+		return 1;
+	}
+
+	return 0;
+}
+
 
 /*
  * PUBLIC
  * Check console buffer.
  * parameters : user buffer,max copy size
- * return : ϥХȿ
+ * return : ϥХȿ or EOF=0 or Ϥʤ=-1
  */
 static int readConsoleBuf(uchar *user_buf,size_t size)
 {
@@ -425,13 +452,22 @@ static int readConsoleBuf(uchar *user_buf,size_t size)
 
 	if(termio.c_lflag&ICANON)	/* Canonical mode. */
 	{
-		if(lbuf.lineTop==0)return 0;
+		if(lbuf.last==0)return -1;		/* Ϥʤ */
+
+		if((uchar)*lbuf.buf==CONSOLE_EOF)
+		{
+			*lbuf.buf='\0';
+			lbuf.lineTop=lbuf.crt=lbuf.last=0;
+
+			return 0;
+		}
 
 		i=0;
-		for(;lbuf.buf[i]!='\n';++i);
+		while (lbuf.buf[i++]!='\n')
+			if (i >= lbuf.last)
+				return -1;
 		size=(size>i)?i:size;
 		memcpy(user_buf,lbuf.buf,size);
-		++i;
 		memcpy(lbuf.buf,lbuf.buf+i,lbuf.last-i);
 		lbuf.crt-=i;
 		lbuf.last-=i;
@@ -446,6 +482,24 @@ static int readConsoleBuf(uchar *user_buf,size_t size)
 }
 
 
+/*
+ * PUBLIC
+ * 󥽡Хåեɤ߹߲ǽ
+ * return : no=0 or yes=1
+ */
+static int checkConsoleBuf()
+{
+	int i;
+
+
+	if((uchar)*lbuf.buf==CONSOLE_EOF)return 1;
+	for(i=0;i<lbuf.last;++i)
+		if(lbuf.buf[i]=='\n')return 1;
+
+	return 0;
+}
+
+
 /***************************************************************************************
  *
  * ܡ
@@ -457,15 +511,16 @@ enum{
 };
 
 
-/* GLOBAL */
-static PROC_LINK waitLink={&waitLink,&waitLink};	/* Ԥץ󥯡 */
+/* PUBLIC */
+static PROC *waitProc;	/* Ԥץ */
+static int waitLock;	/* ѥå */
 
 
 /*
  * PRIVATE
  * Do control command in canonical mode.
  * parameters : character
- * return : 1=ƤӽФȤϽλ or 0
+ * return : 1=ƤӽФȤϽλ of 2=readԤ򵯤 or 0
  */
 static inline int doCanonTermCtr(uchar ch)
 {
@@ -503,6 +558,7 @@ static inline int doCanonTermCtr(uchar ch)
 		}
 		else if(ch==cc[VEOF])		/* եνλ */
 		{
+			if(Eof())return 2;
 			return 0;
 		}
 		else if(ch==cc[VEOL])		/* Ԥνλ */
@@ -609,7 +665,11 @@ static int keyboardHnadler(uchar key_coad)
 		/* ȥ륳ޥɤθ */
 		if(map_coad<' ')
 		{
-			if(doCanonTermCtr(map_coad))return 0;
+			switch(doCanonTermCtr(map_coad))
+			{
+				case 1:return 0;
+				case 2:return 1;
+			}
 			switch(map_coad)
 			{
 				case '\r':
@@ -668,12 +728,24 @@ static int keyboardHnadler(uchar key_coad)
  */
 static int keyboardIntrHandler()
 {
+	int rest=0;
+
+
 	if(keyboardHnadler(inb(KEYBOARD_OUTPUT_REG)))
 	{
-		wakeProc(&waitLink);
-		return 1;
+		enter_spinlock(&waitLock);
+		{
+			if(waitProc!=NULL)
+			{
+				forceSendSignal(waitProc,SIGCONT);
+				waitProc=NULL;
+				rest=1;
+			}
+		}
+		exit_spinlock(&waitLock);
 	}
-	return 0;
+
+	return rest;
 }
 
 
@@ -692,31 +764,21 @@ static int keyboardIntrHandler()
 int writeConsole(void *buf,size_t size,size_t start)
 {
 	char *str=(char*)buf;
-	int cursol;
 	int i;
 
 
-	cursol=0;
-	i=0;
-	for(;i<size;++i)
+	for(i=0;i<size;++i)
 	{
 		if(str[i]>=' ')
-		{
 			putIntoScreen(str[i]);
-			++cursol;
-		}
     	else if(str[i]=='\n')
-    	{
     		newLine();
-    		cursol=0;
-    	}
     	else if(str[i]=='\r')
-    	{
     		carriageRetun();
-    		cursol=0;
-    	}
+    	else if(str[i]=='\t')
+    		tab();
     }
-    rightCursor(cursol);
+    rightCursor(0);
 
 	return i;
 }
@@ -728,6 +790,7 @@ int writeConsole(void *buf,size_t size,size_t start)
  *
  ***************************************************************************************/
 
+
 static int open()
 {
 	return 0;
@@ -746,13 +809,17 @@ static int read(void *buf,size_t size,size_t start)
 		wait_task();
 	}
 
-
 	/* Ϥ뤫󥽡Хåեǧ */
-	if((len=readConsoleBuf(buf,size))==0)
+	enter_spinlock(&waitLock);
+	if((len=readConsoleBuf(buf,size))==-1)
 	{
-		waitProc(&waitLink);			/* Ԥ */
+		waitProc=get_current_task();			/* Ԥ */
+		exit_spinlock(&waitLock);
+		sigWait();								/* ȡ */
+
 		len=readConsoleBuf(buf,size);
 	}
+	else exit_spinlock(&waitLock);
 
 	return len;
 }
@@ -771,11 +838,12 @@ static int write(void *buf,size_t count,size_t x)
 }
 
 
-int ioctl(int prm_num,int cmd,...)
+int ioctl(int cmd,...)
 {
 	uint *arg=(uint*)&cmd+1;
 	PROC *proc,*grp_proc;
-	gid_t grp;
+	pid_t pgid;
+	struct winsize *win;
 
 
 	switch(cmd)
@@ -784,33 +852,62 @@ int ioctl(int prm_num,int cmd,...)
 			if(termCtl.ctlProc==NULL)
 			{
 				termCtl.ctlProc=get_current_task();
-				termCtl.foregrnd=termCtl.ctlProc->gid;
+				termCtl.foregrnd=termCtl.ctlProc->pgid;
 				termCtl.ctlProc->ctlterm=&termCtl;
 			}
 			else return -EPERM;
 			break;
 		case TIOSPGRP:
-			grp=(gid_t)*arg;
+			pgid=(gid_t)*arg;
 			proc=get_current_task();
 			if(proc!=termCtl.ctlProc)return -ENOTTY;
-			if(proc->gid!=grp)
+			if(proc->pgid!=pgid)
 			{
 				/* Ϳ줿롼פƱåΤΤ */
-				if((grp_proc=getNextProc(proc,grp))==NULL)return -EINVAL;
+				if((grp_proc=getNextProc(proc,pgid))==NULL)return -EINVAL;
 				if((grp_proc->ctlterm==NULL)||(((TERM_CTL*)grp_proc->ctlterm)->ctlProc!=proc))return -EPERM;
 			}
-			termCtl.foregrnd=grp;
+			termCtl.foregrnd=pgid;
 
 			break;
 		case TIOGPGRP:
 			if(get_current_task()->ctlterm!=&termCtl)return -ENOTTY;
 			return termCtl.foregrnd;
+		case TIOCGWINSZ:
+			win=(struct winsize*)*arg;
+			if(checkMem(win)==-1)return -EFAULT;
+			win->ws_row=SCR_LINES;
+			win->ws_col=SCR_COLUMNS;
+			break;
 	}
 
 	return 0;
 }
 
 
+/* return : 0 or 1=Ѳ */
+static int select(int flag,int polling)
+{
+	int rest;
+
+
+	if(flag==FD_WRITE)return 1;
+
+	/* polling mode. */
+	if(polling)
+		return checkConsoleBuf();
+
+	enter_spinlock(&waitLock);
+	{
+		if((rest=checkConsoleBuf())==0)
+			waitProc=get_current_task();	/* Ԥ */
+	}
+	exit_spinlock(&waitLock);
+
+	return rest;
+}
+
+
 /***************************************************************************************
  *
  * 
@@ -819,7 +916,7 @@ int ioctl(int prm_num,int cmd,...)
 
 /* PRIVATE,Device interface */
 static DEV_INFO devInfo={
-	"console",0,0,0,open,read,write,ioctl,
+	"console",0,0,0,open,read,write,ioctl,select
 };
 
 
