@@ -203,13 +203,15 @@ static DEV_INODE *getNextInode(DEV_INODE *dind)
  * parameters : device inode,stat store buffer
  * return : 0 or error=-1
  */
-int get_dev_stat(int din,DEV_STAT *buf)
+int get_devstat(int din,DEV_STAT *buf)
 {
 	if(dinode[din].ref_count==0)return -1;
 
 	buf->prt_type=dinode[din].pt_type;
+	buf->prt_num=dinode[din].pt_num;
 	buf->sect_size=dinode[din].dev->sector_size;
 	buf->all_sect=dinode[din].blocks;
+	buf->name=dinode[din].dev->name;
 
 	return 0;
 }
@@ -328,13 +330,13 @@ typedef struct CACHE_TABLE{
 	struct CACHE_TABLE *prev;		/* Cache chain previous */
 	struct CACHE_TABLE *write_next;	/* ǥХؽ񤭹߹ꥹ */
 	struct CACHE_TABLE *write_prev; /* ǥХؽ񤭹߹ꥹ */
-	void *addr;						/* Cache address */
-	uint block;						/* Device block index */
-	uchar din;						/* Device inode number */
-	uchar bitmap;					/* ǡ¸ߥӥåȥޥå */
-	uchar dirty;					/* ߤե饰 */
-	uchar volatile writing;			/* ǥХؽ񤭹߹ե饰 */
-	int gate;						/* ԥå */
+	void               *addr;		/* Cache address */
+	uint                block;		/* Device block index */
+	uchar               din;		/* Device inode number */
+	uchar               bitmap;		/* ǡ¸ߥӥåȥޥå */
+	uchar               dirty;		/* ߤե饰 */
+	uchar volatile      writing;	/* ǥХؽ񤭹߹ե饰 */
+	int                 gate;		/* ԥå */
 }CACHE_TABLE;
 
 typedef struct{
@@ -350,6 +352,9 @@ static PROC *write_task=NULL;						/* ٱ񤭹ѥ */
 static CACHE_TABLE delay_list=						/* ٱ񤭹ߥꥹȤƬ */
 	{NULL,NULL,NULL,NULL,&delay_list,&delay_list,NULL,0,0,0,0,0,0};
 static int cacheLock=0;								/* å󥯹ѥåȡ */
+static uchar bitSects[]={							/* PRIVATE ȤΥӥåȥޥå͡ */
+	0,0x1,0x3,0x7,0xf,0x1f,0x3f,0x7f,0xff
+};
 
 
 /*
@@ -358,8 +363,9 @@ static int cacheLock=0;								/* å󥯹ѥåȡ */
  * parameters : CACHE_TABLE *p
  * return : 0 or error number
  */
-static int readDevice(CACHE_TABLE *p)
+static int readDevice(CACHE_TABLE *p,uchar reqmap)
 {
+	uchar map;
 	int i,k;
 
 
@@ -367,23 +373,24 @@ static int readDevice(CACHE_TABLE *p)
 	{
 		if(dinode[p->din].dev->read(p->addr,CACHE_BLOCK_SIZE/dinode[p->din].dev->sector_size,p->block+dinode[p->din].begin_blk)<0)
 			return  -EIO;
+		p->bitmap=0xff;
 	}
 	else
 	{
 		/* ɤ߹ޤƤʤս򥭥åɤ߹ࡣ */
+		map=~p->bitmap&reqmap;
 		for(i=0;i<CACHE_BLOCK_SIZE/CACHE_UNIT_SIZE;++i)
-			if((p->bitmap&(1<<i))==0)
+			if((map&(1<<i))!=0)
 			{
 				for(k=i+1;k<CACHE_BLOCK_SIZE/CACHE_UNIT_SIZE;++k)
-					if((p->bitmap&(1<<k))!=0)break;
+					if((map&(1<<k))==0)break;
 				if(dinode[p->din].dev->read((char*)p->addr+i*CACHE_UNIT_SIZE,k-i,p->block+i+dinode[p->din].begin_blk)<0)
 					return -EIO;
 				i=k;
 			}
+		p->bitmap|=reqmap;
 	}
 
-	p->bitmap=0xff;
-
 	return 0;
 }
 
@@ -540,16 +547,19 @@ static CACHE_TABLE *search_cache(int din,size_t begin)
 			if(p==cache_old)
 			{
 				cache_old=p->prev;
-				if(cache_old->dirty)add_delay_list(cache_old);
+				add_delay_list(cache_old);
 			}
-
-			/* 󥯤ֿ֤ˤ롣 */
-			p->next->prev=p->prev;
-			p->prev->next=p->next;
-			p->next=cache_old->next;
-			p->prev=cache_old;
-			cache_old->next=p;
-			p->next->prev=p;
+			else
+			{
+				/* 󥯤ֿ֤ˤ롣 */
+				p->next->prev=p->prev;
+				p->prev->next=p->next;
+				p->next=cache_old->next;
+				p->prev=cache_old;
+				cache_old->next=p;
+				p->next->prev=p;
+			}
+			add_delay_list(cache_old->prev);
 
 			return p;
 		}
@@ -576,7 +586,7 @@ static CACHE_TABLE *getEmptyCache(int din,size_t begin)
 	cache_old=p->prev;
 
 	/* μθŤå᤹ */
-	if(cache_old->prev->dirty)add_delay_list(cache_old->prev);
+	add_delay_list(cache_old->prev);
 
 	/* ϥå󥯤 */
 	p->hprev->hnext=p->hnext;
@@ -644,42 +654,53 @@ static void writeBackAllCache()
  */
 int read_cache(int din,void *buf,size_t sector_num,size_t begin)
 {
-	int sector_size=dinode[din].dev->sector_size;	/* ǥХ */
-	int sectors=CACHE_BLOCK_SIZE/sector_size;		/* å֥åˤϤ륻 */
-	int cache_mask=sectors-1;						/* åƬ򻻽Ф뤿Υӥåȥޥ */
-	int rest_num=sector_num;						/* Ĥž */
+	int sector_size = dinode[din].dev->sector_size;	/* ǥХ */
+	int sectors = CACHE_BLOCK_SIZE / sector_size;	/* å֥åˤϤ륻 */
+	int cache_mask = sectors - 1;					/* åƬ򻻽Ф뤿Υӥåȥޥ */
+	int rest_num = sector_num;						/* Ĥž */
 	int start,sect_num;
 	CACHE_TABLE *p;
+	union{
+		void *pv;
+		uint ui;
+	}value;
 
 
 	/* ǥ¤Υå */
-	if(begin>dinode[din].blocks)return -EIO;
-	if(begin+sectors>dinode[din].blocks)return -EIO;
+	if (begin > dinode[din].blocks)
+		return -EIO;
+	if (begin + sectors > dinode[din].blocks)
+		return -EIO;
 
-	start=begin&cache_mask;
-	begin&=~cache_mask;
-	sect_num=(rest_num<sectors-start)?rest_num:sectors-start;
+	start = begin & cache_mask;
+	begin &= ~cache_mask;
+	sect_num = (rest_num < sectors - start)? rest_num : sectors - start;
 	for(;;)
 	{
 		/* å򸡺 */
 		enter_spinlock(&cacheLock);
 		{
-			if((p=search_cache(din,begin))==NULL)p=getEmptyCache(din,begin);
+			if ((p = search_cache(din,begin)) == NULL)
+				p = getEmptyCache(din,begin);
 		}
 		exit_spinlock(&cacheLock);
 
 		/* å׵᥻ʤХǥХɤ߹ࡣ */
-		if((p->bitmap|(((1<<sect_num)-1)<<start))!=p->bitmap)
-			if(readDevice(p)<0)return -EIO;
+		if (((p->bitmap >> start) & bitSects[sect_num]) != bitSects[sect_num])
+			if (readDevice(p,bitSects[sect_num] << start) < 0)
+				return -EIO;
 
 		/* Хåե˥ԡ롣 */
 		memcpy(buf,(char*)p->addr+start*sector_size,sect_num*sector_size);
 
-		if((rest_num-=sect_num)==0)break;
-		(char*)buf+=sect_num*sector_size;
-		begin+=sectors;
-		sect_num=(rest_num<sectors)?rest_num:sectors;
-		start=0;
+		if ((rest_num -= sect_num) == 0)
+			break;
+		value.pv = buf;
+		value.ui += sect_num * sector_size;
+		buf = value.pv;
+		begin += sectors;
+		sect_num = (rest_num < sectors)? rest_num : sectors;
+		start = 0;
 	}
 
 	return sector_num;
@@ -694,41 +715,51 @@ int read_cache(int din,void *buf,size_t sector_num,size_t begin)
  */
 int write_cache(int din,void *buf,size_t sector_num,size_t begin)
 {
-	int sector_size=dinode[din].dev->sector_size;	/* ǥХ */
-	int sectors=CACHE_BLOCK_SIZE/sector_size;		/* å֥åˤϤ륻 */
-	int cache_mask=sectors-1;						/* åƬ򻻽Ф뤿Υӥåȥޥ */
-	int rest_num=sector_num;						/* Ĥž */
+	int sector_size = dinode[din].dev->sector_size;	/* ǥХ */
+	int sectors = CACHE_BLOCK_SIZE / sector_size;	/* å֥åˤϤ륻 */
+	int cache_mask = sectors - 1;					/* åƬ򻻽Ф뤿Υӥåȥޥ */
+	int rest_num = sector_num;						/* Ĥž */
 	int start,sect_num;
 	CACHE_TABLE *p;
+	union{
+		void *pv;
+		uint ui;
+	}value;
 
 
 	/* ǥ¤Υå */
-	if(begin>dinode[din].blocks)return -EIO;
-	if(begin+sectors>dinode[din].blocks)return -EIO;
+	if (begin > dinode[din].blocks)
+		return -EIO;
+	if (begin + sectors > dinode[din].blocks)
+		return -EIO;
 
-	start=begin&cache_mask;
-	begin&=~cache_mask;
-	sect_num=(rest_num<sectors-start)?rest_num:sectors-start;
+	start = begin & cache_mask;
+	begin &= ~cache_mask;
+	sect_num = (rest_num < sectors - start)? rest_num : sectors-start;
 	for(;;)
 	{
 		/* å򸡺 */
 		enter_spinlock(&cacheLock);
 		{
-			if((p=search_cache(din,begin))==NULL)p=getEmptyCache(din,begin);
+			if ((p = search_cache(din,begin)) == NULL)
+				p = getEmptyCache(din,begin);
 		}
 		exit_spinlock(&cacheLock);
 
 		/* å˥ԡ롣 */
-		memcpy((char*)p->addr+start*sector_size,buf,sect_num*sector_size);
+		memcpy((char*)p->addr + start * sector_size,buf,sect_num * sector_size);
 
-		p->bitmap|=((1<<sect_num)-1)<<start;
-		p->dirty=1;
+		p->bitmap |= bitSects[sect_num] << start;
+		p->dirty = 1;
 
-		if((rest_num-=sect_num)==0)break;
-		(char*)buf+=sect_num*sector_size;
-		begin+=sectors;
-		sect_num=(rest_num<sectors)?rest_num:sectors;
-		start=0;
+		if ((rest_num -= sect_num) == 0)
+			break;
+		value.pv = buf;
+		value.ui += sect_num * sector_size;
+		buf = value.pv;
+		begin += sectors;
+		sect_num = (rest_num < sectors)? rest_num : sectors;
+		start = 0;
 	}
 
 	return sector_num;
@@ -845,7 +876,6 @@ static int open(const char *path,int dev,uint dir,int flag,uint *inode)
 	int i;
 
 
-
 	/* ץγǧ */
 	if(isMaxDevOpen()==0)return -EMDEV;
 
@@ -906,22 +936,24 @@ static int open(const char *path,int dev,uint dir,int flag,uint *inode)
 
 		/* ĥѡƥ󥯤򤿤ɤ롣 */
 		beg_blk=next_blk=pt[i].lba_begin;
-		for(i=PT_NUM;;)
+		for(i = PT_NUM;;)
 		{
-			if((rest=dinf->read(buf,1,next_blk))<0)goto ERR;
-			if(++i==pt_num)break;
-			if(pt[1].type!=PT_TYPE_EXT)
+			if ((rest = dinf->read(buf,1,next_blk)) < 0)
+				goto ERR;
+			if (++i == pt_num)
+				break;
+			if (pt[1].type != PT_TYPE_EXT)
 			{
-				rest=-ENODEV;
+				rest = -ENODEV;
 				goto ERR;
 			}
-			next_blk=beg_blk+pt[1].lba_begin;
+			next_blk = beg_blk+pt[1].lba_begin;
 		}
-		dind=makeInode(name,pt[0].lba_begin+next_blk,pt[0].size,pt[0].type,pt_num,dinf);
+		dind = makeInode(name,pt[0].lba_begin+next_blk,pt[0].size,pt[0].type,pt_num,dinf);
 	}
 	kfree(buf);
-	dind->ref_count=1;
-	*inode=getInodeNum(dind);
+	dind->ref_count = 1;
+	*inode = getInodeNum(dind);
 
 	return DEVICE_FILE;
 ERR:
@@ -933,7 +965,7 @@ ERR:
 /*
  * parameters : device inode,inode block
  */
-static int close(int din,uint inode)
+static int close(int din,uint inode,int type)
 {
 	if(--dinode[inode].ref_count<0)dinode[inode].ref_count=0;
 
@@ -950,6 +982,10 @@ static int read(int din,uint inode,void *buf,size_t size,size_t begin)
 	int rest=size;		/* ĤžХȿ */
 	int beg_sect;		/* žϤΥֹ */
 	int s,t;
+	union{
+		void *pv;
+		uint ui;
+	}value;
 
 
 	if((sect_siz=dinode[inode].dev->sector_size)==0)
@@ -964,7 +1000,7 @@ static int read(int din,uint inode,void *buf,size_t size,size_t begin)
 	if((s=begin%sect_siz)!=0)
 	{
 		if((tmp=(char*)kmalloc(sect_siz))==NULL)return -1;
-		if(read_cache(din,tmp,1,beg_sect)!=1)return 0;
+		if(read_cache(inode,tmp,1,beg_sect)!=1)return 0;
 		t=sect_siz-s;
 		if(t>size)
 		{
@@ -973,13 +1009,15 @@ static int read(int din,uint inode,void *buf,size_t size,size_t begin)
 			return size;
 		}
 		memcpy(buf,tmp+s,t);
-		(char*)buf+=t;
+		value.pv = buf;
+		value.ui += t;
+		buf = value.pv;
 		beg_sect+=1;
 		rest-=t;
 	}
 
 	s=rest/sect_siz;
-	if((t=read_cache(din,buf,s,beg_sect))<s)
+	if((t=read_cache(inode,buf,s,beg_sect))<s)
 	{
 		kfree(tmp);
 		return size-rest+t*sect_siz;
@@ -990,7 +1028,7 @@ static int read(int din,uint inode,void *buf,size_t size,size_t begin)
 	{
 		t=s*sect_siz;
 		if((tmp==NULL)&&((tmp=(char*)kmalloc(sect_siz))==NULL))return -1;
-		if(read_cache(din,tmp,1,beg_sect+s)!=1)
+		if(read_cache(inode,tmp,1,beg_sect+s)!=1)
 		{
 			kfree(tmp);
 			return size-rest+t;
@@ -1013,6 +1051,11 @@ static int write(int din,uint inode,void *buf,size_t size,size_t begin)
 	int rest=size;		/* ĤžХȿ */
 	int beg_sect;		/* žϤΥֹ */
 	int s,t;
+	union{
+		void *pv;
+		uint ui;
+	}value;
+
 
 
 	if((sect_siz=dinode[inode].dev->sector_size)==0)
@@ -1027,23 +1070,25 @@ static int write(int din,uint inode,void *buf,size_t size,size_t begin)
 	if((s=begin%sect_siz)!=0)
 	{
 		if((tmp=(char*)kmalloc(sect_siz))==NULL)return -1;
-		if(read_cache(din,tmp,1,beg_sect)!=1)return 0;
+		if(read_cache(inode,tmp,1,beg_sect)!=1)return 0;
 		if((t=sect_siz-s)>size)
 		{
 			memcpy(tmp+s,buf,size);
-			if(write_cache(din,tmp,1,beg_sect)!=1)return 0;
+			if(write_cache(inode,tmp,1,beg_sect)!=1)return 0;
 			kfree(tmp);
 			return size;
 		}
 		memcpy(tmp+s,buf,t);
-		if(write_cache(din,tmp,1,beg_sect)!=1)return 0;
-		(char*)buf+=t;
+		if(write_cache(inode,tmp,1,beg_sect)!=1)return 0;
+		value.pv = buf;
+		value.ui += t;
+		buf = value.pv;
 		beg_sect+=1;
 		rest-=t;
 	}
 
 	s=rest/sect_siz;
-	if((t=write_cache(din,buf,s,beg_sect))<s)
+	if((t=write_cache(inode,buf,s,beg_sect))<s)
 	{
 		kfree(tmp);
 		return size-rest+t*sect_siz;
@@ -1054,13 +1099,13 @@ static int write(int din,uint inode,void *buf,size_t size,size_t begin)
 	{
 		t=s*sect_siz;
 		if((tmp==NULL)&&((tmp=(char*)kmalloc(sect_siz))==NULL))return -1;
-		if(read_cache(din,tmp,1,beg_sect+s)!=1)
+		if(read_cache(inode,tmp,1,beg_sect+s)!=1)
 		{
 			kfree(tmp);
 			return size-rest+t;
 		}
 		memcpy(tmp,(char*)buf+t,rest-t);
-		if(write_cache(din,tmp,1,beg_sect+s)!=1)
+		if(write_cache(inode,tmp,1,beg_sect+s)!=1)
 		{
 			kfree(tmp);
 			return size-rest+t;
@@ -1073,7 +1118,7 @@ static int write(int din,uint inode,void *buf,size_t size,size_t begin)
 }
 
 
-static int opendir(const char *path,int din,uint dir_inode,DIR *dir)
+static int opendir(const char *path,int din,uint dir_inode,uint *inode,uint *block)
 {
 	const char *next;
 
@@ -1081,7 +1126,7 @@ static int opendir(const char *path,int din,uint dir_inode,DIR *dir)
 	for(;(next=cmpPathNull(".",path))!=NULL;path=next);
 	if(*path!='\0')return -ENOENT;
 
-	dir->index=0;
+	*inode=0;
 
 	return 0;
 }
@@ -1102,13 +1147,13 @@ static inline void itoa(int v,char *buf)
 	while(i>=0)*buf++=tmp[i--];
 }
 
-static int readdir(int din,uint inode,DIR *dir,char *name)
+static int readdir(int din,uint inode,uint *block,uint *index,char *name)
 {
 	char tmp[4];
 	DEV_INODE *dind;
 
 
-	if((dind=getNextInode((DEV_INODE*)dir->index))==NULL)
+	if((dind=getNextInode((DEV_INODE*)*index))==NULL)
 		*name='\0';
 	else
 	{
@@ -1118,26 +1163,28 @@ static int readdir(int din,uint inode,DIR *dir,char *name)
 			itoa(dind->pt_num,tmp);
 			strcat(name,tmp);
 		}
-		dir->index=(uint)dind;
+		*index=(uint)dind;
 	}
 
 	return 0;
 }
 
 
-static int stat(int din,uint inode,FSTAT *fstat)
+static int stat(int din,uint inode,int type,FSTAT *fstat)
 {
-	fstat->st_dev=inode;
-	fstat->st_ino=0;
-	fstat->st_mode=0;
-	fstat->st_nlink=0;
-	fstat->st_uid=0;
-	fstat->st_gid=0;
-	fstat->st_size=dinode[inode].blocks*dinode[inode].dev->sector_size;
-	fstat->st_atime=0;
-	fstat->st_mtime=0;
-	fstat->st_ctime=0;
+	fstat->st_dev=    inode;
+	fstat->st_ino=    inode;	/* ȤꤢinodeƤ */
+	fstat->st_mode=   0;
+	fstat->st_nlink=  0;
+	fstat->st_uid=    0;
+	fstat->st_gid=    0;
+	fstat->st_size=   dinode[inode].blocks*dinode[inode].dev->sector_size;
+	fstat->st_atime=  0;
+	fstat->st_mtime=  0;
+	fstat->st_ctime=  0;
 	fstat->st_blksize=dinode[inode].dev->sector_size;
+	fstat->st_blocks= 0;
+	fstat->st_rdev=   0;
 
 	return 0;
 }
@@ -1154,11 +1201,11 @@ static int ioctl(int din,uint inode,int prm_num,int cmd,...)
 	switch(prm_num)
 	{
 		case 0:
-			return dinode[inode].dev->ioctl(0,cmd);
+			return dinode[inode].dev->ioctl(cmd);
 		case 1:
-			return dinode[inode].dev->ioctl(1,cmd,*arg);
+			return dinode[inode].dev->ioctl(cmd,*arg);
 		case 2:
-			return dinode[inode].dev->ioctl(2,cmd,*arg,*(arg+1));
+			return dinode[inode].dev->ioctl(cmd,*arg,*(arg+1));
 		default:
 			return 0;
 	}
@@ -1183,7 +1230,7 @@ static int rename(int din,uint old_dinode,const char *old_path,uint new_dinode,c
 }
 
 
-static int creat(const char *path,int din,uint dir_inode,int mode)
+static int creat(const char *path,int din,uint dir_inode,int mode,uint *inode)
 {
 	return -EDNOFUNC;
 }
@@ -1200,7 +1247,7 @@ static int delete(const char *path,int din,uint dir_inode,int type)
 	return -EDNOFUNC;
 }
 
-static int chattr(int din,uint inode,uint mode,uint uid,uint gid,uint atime,uint mtime)
+static int chattr(int din,uint inode,int type,uint mode,uint uid,uint gid,uint atime,uint mtime)
 {
 	return -EDNOFUNC;
 }
@@ -1211,9 +1258,22 @@ static int mklink(int din,uint oldinode,uint newind,const char *newpath)
 }
 
 
+static int select(uint inode,int flag,int polling)
+{
+	return dinode[inode].dev->select(flag,polling);
+}
+
+
+static int fsstat(int din,struct statfs *buf)
+{
+	return -EDNOFUNC;
+}
+
+
 static FS dev_fs={
 	"dev_fs",mount,umount,open,close,read,write,ioctl,rename,creat,opendir,mkdir,stat,delete,readdir,
-	chattr,mklink};
+	chattr,mklink,select,fsstat,
+};
 
 
 /*
@@ -1239,8 +1299,8 @@ int sys_ttyname(int fdnum,char *buf)
 	if((fd=getFd(fdnum))==NULL)return -EBADF;
 
 	/* DEV_INFOΥ饹ȥ֥åʥС0Υ饯ǥХǧ롣 */
-	dev=dinode[fd->vent->din].dev;
-	if(dev->last_blk!=0)return -ENOTTY;
+	dev=dinode[fd->vent->inode].dev;
+	if(dev->sector_size!=0)return -ENOTTY;
 
 	memcpy(buf,dev->name,strlen(dev->name)+1);
 
