@@ -12,11 +12,14 @@
 
 #include<config.h>
 #include<types.h>
-#include<proc.h>
+#include<wait.h>
 #include<share/fcntl.h>
 #include<share/unistd.h>
 #include<share/stat.h>
-#include<share/dirent.h>
+#include<share/select.h>
+#include<share/stropts.h>
+#include<sys/poll.h>
+#include<sys/statvfs.h>
 
 
 #ifndef ASM_FILE
@@ -31,8 +34,14 @@ enum{
 	FIFO_FILE,
 
 	/* եǥץѥեμࡣ */
-	FD_FORM_LOCALFILE=0,
+	FD_FORM_LOCALFILE=1,
+	FD_FORM_DIRSTREAM,
 	FD_FORM_NETSOKET,
+
+	/* select()fd̡ */
+	FD_READ= 1,	/* Read discriptor. */
+	FD_WRITE=2,	/* Write discriptor. */
+	FD_ERR=  3,	/* Except watch. */
 };
 
 
@@ -42,41 +51,57 @@ typedef struct stat FSTAT;
 /* ե륷ƥ।ե᡼¤ */
 typedef struct FS{
 	const char *name;
-	uint (*mount)(int);							/* prm:device inode ret:root directory block or error=0 */
-	int (*umount)(int);							/* prm:device inode ret:0 or error=-1 */
-	int (*open)(const char*,int,uint,int,uint*);/* prm:path,device inode,directory inode,flag,inode buffer ret:file format or error */
-	int (*close)(int,uint);						/* prm:device inode,inode ret:0 or error=-1 */
-	int (*read)(int,uint,void*,size_t,size_t);	/* prm:device inode,inode,bffer,read bytes,begin byte ret:read bytes or error=-1 */
-	int (*write)(int,uint,void*,size_t,size_t);	/* prm:device param num,inode,bffer,write bytes,begin byte ret:write bytes or error=-1 */
-	int (*ioctl)(int,uint,int,int,...);			/* prm:device inode,inode,number of parameters,command,parameter ret:0 or error=-1 */
+	/* prm:device inode ret:root directory block or error=0 */
+	uint (*mount)(int);
+	/* prm:device inode ret:0 or error=-1 */
+	int (*umount)(int);
+	/* prm:path,device inode,directory inode,flag,inode buffer ret:file format or error */
+	int (*open)(const char*,int,uint,int,uint*);
+	/* prm:device inode,inode,type ret:0 or error=-1 */
+	int (*close)(int,uint,int);
+	/* prm:device inode,inode,bffer,read bytes,begin byte ret:read bytes or error=-1 */
+	int (*read)(int,uint,void*,size_t,size_t);
+	/* prm:device param num,inode,bffer,write bytes,begin byte ret:write bytes or error=-1 */
+	int (*write)(int,uint,void*,size_t,size_t);
+	/* prm:device inode,inode,number of parameters,command,parameter ret:0 or error=-1 */
+	int (*ioctl)(int,uint,int,int,...);
 	/* prm:device inode,old dir inode,old path,new dir inode,new path ret:0 or error number */
 	int (*rename)(int,uint,const char*,uint,const char*);
-	int (*creat)(const char*,int,uint,int);		/* prm:path,device inode,directory inode,mode ret:directory block or error=0 */
-	int (*opendir)(const char*,int,uint,DIR*);	/* prm:path,device inode,directory inode,DIR struct ret:inode block or error number */
-	int (*mkdir)(const char*,int,uint,int);		/* prm:path,device inode,parent directory block,mode ret:directory block or error=0 */
-	int (*stat)(int,uint,FSTAT*);				/* prm:device inode,directory inode of inode,FSTAT buffer ret:0 or error number */
-	int (*delete)(const char*,int,uint,int);	/* prm:path,device inod,directory block,file type ret:0 or error number */
-	int (*readdir)(int,uint,DIR*,char*);		/* prm:device inode,inode,DIR struct,name buffer ret:0 or error number */
+	/* prm:path,device inode,directory inode,mode,inode buffer ret:directory block or error=0 */
+	int (*creat)(const char*,int,uint,int,uint*);
+	/* prm:path,device inode,directory inode,inodebuffer,block buffer:0 or error number */
+	int (*opendir)(const char*,int,uint,uint*,uint*);
+	/* prm:path,device inode,parent directory block,mode ret:directory block or error=0 */
+	int (*mkdir)(const char*,int,uint,int);
+	/* prm:device inode,inode,filr format,FSTAT buffer ret:0 or error number */
+	int (*stat)(int,uint,int,FSTAT*);
+	/* prm:path,device inod,directory block,file type ret:0 or error number */
+	int (*delete)(const char*,int,uint,int);
+	/* prm:device inode,inode,block buffer,index buffer,name buffer ret:0 or error number */
+	int (*readdir)(int,uint,uint*,uint*,char*);
 	/* prm:device inode,directory inode or inode,mode,uid,gid,access time,modified time ret:0 or error number */
-	int (*chattr)(int,uint,uint,uint,uint,uint,uint);
+	int (*chattr)(int,uint,int,uint,uint,uint,uint,uint);
 	int (*mklink)(int,uint,uint,const char*);
+	/* param:inode,fdμ,polling flag  return:o=no or 1=yes */
+	int (*select)(uint,int,int);
+	/* param:device inode  return:0  */
+	int (*fsstat)(int din,struct statfs *buf);
 	struct FS *next;
 }FS;
 
 /*
  * ۥȥ꡼
  * Note!
- *  fsblockΰ֤ϥåȹ¤Τȥ󥯤ƤΤա
+ *  fsinodeΰ֤ϥåȹ¤Τȥ󥯤ƤΤա
  */
 typedef struct VENTRY{
 	FS            *fs;			/* Filesystem. */
-	uint           block;		/* ǥХ֥å */
+	uint           inode;		/* inode */
 	struct VENTRY *next;		/* 饦ɥ󥯥ꥹȥͥ */
 	struct VENTRY *prev;		/* 饦ɥ󥯥ꥹȥץӥ塼 */
 	struct VENTRY *parent;		/* ƥȥ꡼ */
 	struct VENTRY *child;		/* ΥΡɤΥȥåץȥ꡼ */
 	struct VENTRY *mount;		/* ޥȥȥ꡼ */
-	int            busy;		/* busy flag. */
 	WAIT_QUEUE    *wait;		/* Lock wait. */
 	uchar          refCount;	/* ȥ󥿡ޥȤ䥫ȥǥ쥯ȥ¹ԥե¹ԥ */
 	uchar          din;			/* ǥХinode */
@@ -94,6 +119,13 @@ typedef struct{
 	VENTRY *vent;		/* ۥȥ꡼ */
 }F_DSC;
 
+/* ǥ쥯ȥꥹȥ꡼ࡣ */
+typedef struct{
+	F_DSC fd;
+	uint  block;		/* directory block. */
+	uint  index;		/* directory index. */
+}DIR_STRAEM;
+
 /* ץѥե빽¤ */
 typedef struct{
 	VENTRY *current_dir;		/* ȥǥ쥯ȥꡣ */
@@ -258,14 +290,21 @@ extern int sys_umount();
 extern int sys_opendir();
 extern int sys_closedir();
 extern int sys_readdir();
+extern int sys_rewinddir();
 extern int sys_chdir();
+extern int sys_fchdir();
 extern int sys_getcwd();
 extern int sys_stat();
 extern int sys_chattr();
+extern int sys_fchattr();
 extern int sys_umount_root();
 extern int sys_fcntl();
 extern int sys_ioctl();
 extern int sys_link();
+extern int sys_select();
+extern int sys_poll();
+extern int sys_getfsstat();
+extern int sys_fstatvfs();
 
 
 #endif
