@@ -9,7 +9,6 @@
 #include<lib.h>
 #include<proc.h>
 #include<segment.h>
-#include<interrupt.h>
 #include<mp.h>
 #include<fs.h>
 #include<time.h>
@@ -20,6 +19,7 @@
 #include<net/net.h>
 #include<setup.h>
 #include<test.h>
+#include<interrupt.h>
 
 
 enum{IDT_ENTRY=256};		/* Interrupt descriptorȥ꡼ */
@@ -87,6 +87,57 @@ void set_gate(int des,void *addr,uchar para_num)
 }
 
 
+/*
+ * GLOBAL 
+ * ̾ߤΥʥ¹ԡ
+ * parameters : user esp
+ */
+void doSignalFromTrap(TRAP_FRAME frame)
+{
+	doSignal((uint*)&frame.user_esp);
+}
+
+void doSignalFromExcept(EXCEPT_FRAME frame)
+{
+	doSignal((uint*)&frame.user_esp);
+}
+
+void doSignalFromSyscall1(int eax,SYSCALL1_FRAME frame)
+{
+	doSignal((uint*)&frame.user_esp);
+}
+
+void doSignalFromSyscall2(int eax,SYSCALL2_FRAME frame)
+{
+	doSignal((uint*)&frame.user_esp);
+}
+
+void doSignalFromSyscall3(int eax,SYSCALL3_FRAME frame)
+{
+	doSignal((uint*)&frame.user_esp);
+}
+
+void doSignalFromSyscall4(int eax,SYSCALL4_FRAME frame)
+{
+	doSignal((uint*)&frame.user_esp);
+}
+
+void doSignalFromSyscall5(int eax,SYSCALL5_FRAME frame)
+{
+	doSignal((uint*)&frame.user_esp);
+}
+
+void doSignalFromSyscall6(int eax,SYSCALL6_FRAME frame)
+{
+	doSignal((uint*)&frame.user_esp);
+}
+
+void doSignalFromSyscall7(int eax,SYSCALL7_FRAME frame)
+{
+	doSignal((uint*)&frame.user_esp);
+}
+
+
 /*
  * Init system call
  */
@@ -103,9 +154,10 @@ int init_syscall()
 	syscall_table[SYS_UNLINK]=sys_unlink;
 	syscall_table[SYS_RMDIR]=sys_rmdir;
 	syscall_table[SYS_OPENDIR]=sys_opendir;
-	syscall_table[SYS_CLOSEDIR]=sys_closedir;
 	syscall_table[SYS_READDIR]=sys_readdir;
+	syscall_table[SYS_REWINDDIR]=sys_rewinddir;
 	syscall_table[SYS_CHDIR]=sys_chdir;
+	syscall_table[SYS_FCHDIR]=sys_fchdir;
 	syscall_table[SYS_GETCWD]=sys_getcwd;
 	syscall_table[SYS_STAT]=sys_stat;
 	syscall_table[SYS_LOCK]=sys_lock;
@@ -117,6 +169,7 @@ int init_syscall()
 	syscall_table[SYS_SLEEP]=sys_sleep;
 	syscall_table[SYS_ALARM]=sys_alarm;
 	syscall_table[SYS_TIME]=sys_time;
+	syscall_table[SYS_GETTIMEOFDAY]=sys_gettimeofday;
 	syscall_table[SYS_SYNC]=sys_sync;
 	syscall_table[SYS_WAIT]=sys_wait;
 	syscall_table[SYS_SET_PROCID]=sys_set_procid;
@@ -132,10 +185,16 @@ int init_syscall()
 	syscall_table[SYS_REBOOT]=sys_reboot;
 	syscall_table[SYS_FCNTL]=sys_fcntl;
 	syscall_table[SYS_CHATTR]=sys_chattr;
+	syscall_table[SYS_FCHATTR]=sys_fchattr;
 	syscall_table[SYS_IOCTL]=sys_ioctl;
 	syscall_table[SYS_LINK]=sys_link;
+	syscall_table[SYS_SELECT]=sys_select;
+	syscall_table[SYS_POLL]=sys_poll;
+	syscall_table[SYS_GETFSSTAT]=sys_getfsstat;
+	syscall_table[SYS_FSTATVFS]=sys_fstatvfs;
 	syscall_table[SYS_UNAME]=sys_uname;
 	syscall_table[SYS_TTYNAME]=sys_ttyname;
+	syscall_table[SYS_SETTIME]=sys_settime;
 	syscall_table[SYS_SOCKET]=sys_socket;
 	syscall_table[SYS_SEND]=sys_send;
 	syscall_table[SYS_RECV]=sys_recv;
@@ -144,18 +203,23 @@ int init_syscall()
 	syscall_table[SYS_BIND]=sys_bind;
 	syscall_table[SYS_LISTEN]=sys_listen;
 	syscall_table[SYS_ACCEPT]=sys_accept;
+	syscall_table[SYS_GET_SOCKADDR]=sys_getsockaddr;
+	syscall_table[SYS_GETSOCKOPT]=sys_getsockopt;
+	syscall_table[SYS_SETSOCKOPT]=sys_setsockopt;
+	syscall_table[SYS_SENDMSG]=sys_sendmsg;
+	syscall_table[SYS_RECVMSG]=sys_recvmsg;
 /*************************************************/
 	syscall_table[TEST_EXT2]=test_ext2;
 	syscall_table[SYS_TEST]=test_mm;
 /*************************************************/
 
-	set_gate(GATE_DES,syscall1,1);
-	set_gate(GATE_DES+8,syscall2,2);
-	set_gate(GATE_DES+16,syscall3,3);
-	set_gate(GATE_DES+24,syscall4,4);
-	set_gate(GATE_DES+32,syscall5,5);
-	set_gate(GATE_DES+40,syscall6,6);
-	set_gate(GATE_DES+48,syscall7,7);
+	set_gate(SYSCALL0_SEG,syscall0,1);
+	set_gate(SYSCALL1_SEG,syscall1,2);
+	set_gate(SYSCALL2_SEG,syscall2,3);
+	set_gate(SYSCALL3_SEG,syscall3,4);
+	set_gate(SYSCALL4_SEG,syscall4,5);
+	set_gate(SYSCALL5_SEG,syscall5,6);
+	set_gate(SYSCALL6_SEG,syscall6,7);
 
 	return 0;
 }
