@@ -33,6 +33,7 @@
 #include"device.h"
 #include"signal.h"
 #include"mm.h"
+#include"debug.h"
 #include"test.h"
 
 
@@ -258,6 +259,34 @@ static void unlinkPhysTable(uint addr)
 }
 
 
+/*
+ * PUBLIC
+ * ȥ󥿤1餹1ξ-1֤
+ * parameters : ʪɥ쥹
+ * return : 0 or -1
+ */
+static int delPhysRefCount(uint addr)
+{
+	int rest;
+	PHYS_MEM_TABL *mtbl;
+
+
+	mtbl=getPhysTableFromAddr((void*)addr);
+	enter_spinlock(&physMemGate);
+	{
+		if(mtbl->ref_count>1)
+		{
+			--mtbl->ref_count;
+			rest=0;
+		}
+		else rest=-1;
+	}
+	exit_spinlock(&physMemGate);
+
+	return rest;
+}
+
+
 /*
  * PUBLIC
  * ֤ͥʪڡƤ롣
@@ -483,39 +512,46 @@ static uint findAllMemsize()
  *
  ***********************************************************************************/
 
-typedef struct HEAP_MEM{
-	char* last;
-	struct HEAP_MEM *next;
-}HEAP_MEM;
+#define HEAP_NUM() (sizeof(heapHead)/sizeof(HEAP_HEAD))
+
+/* ʬ꡼إå */
+typedef struct MEM_HEAD{
+	struct MEM_HEAD *next;	/* ζ꡼Ƭ */
+	int size;				/* ƥ */
+}MEM_HEAD;
 
 /*
- * Note!
- *  "mem"HEAP_MEM"next"ƱեåȰ֤֤ȡ
+ * ҡץإå
+ * Note! : "memTop"ΰ֤ɬ"MEM_HEAD""next"Ʊˤ뤳
  */
 typedef struct{
-	int dummy;		/* HEAP_MEM"last"Ʊեåȡkfree()HEAP_MEMȸʤȤɬס */
-	HEAP_MEM *mem;
-	uint size;
-	int gate;
-}HEAP;
+	MEM_HEAD *memTop;		/* ꡼Ƭ */
+	uint size;				/* ƥ */
+	int gate;				/* ԥåѥ */
+}HEAP_HEAD;
 
 
-/* PRIVATE kmallocѹ¤Ρ */
-static HEAP heap[]={
-	{0,NULL,0,     0},
-	{0,NULL,0x10,  0},
-	{0,NULL,0x20,  0},
-	{0,NULL,0x40,  0},
-	{0,NULL,0x80,  0},
-	{0,NULL,0x100, 0},
-	{0,NULL,0x200, 0},
-	{0,NULL,0x400, 0},
-	{0,NULL,0x800, 0},
-	{0,NULL,0x1000,0},
+/*
+ * PRIVATE
+ * ҡץإå
+ */
+static HEAP_HEAD heapHead[]={
+	{NULL,0,     0},
+	{NULL,0x10,  0},
+	{NULL,0x20,  0},
+	{NULL,0x40,  0},
+	{NULL,0x80,  0},
+	{NULL,0x100, 0},
+	{NULL,0x200, 0},
+	{NULL,0x400, 0},
+	{NULL,0x800, 0},
+	{NULL,0x1000,0},
 };
 
-
-/* PRIVATE,Хͤҡ׹¤ΥʥС롣 */
+/*
+ * PRIVATE
+ * Хͤҡ׹¤ΥʥС
+ */
 static uchar heapNum1[]={
 	0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
 	1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
@@ -534,16 +570,12 @@ static uchar heapNum1[]={
 	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
 	5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
 };
-
 static uchar heapNum2[]={
 	0,5,6,7,7,8,8,8,8,9,9,9,9,9,9,9,
 	9,
 };
 
 
-#define HEAP_NUM() (sizeof(heap)/sizeof(HEAP))
-
-
 /*
  * PRIVATE
  * ҡץ롣
@@ -552,7 +584,7 @@ static uchar heapNum2[]={
  */
 static inline int getHeapSize(size_t size)
 {
-	if(size>0x100)
+	if(size>=0x100)
 		return heapNum2[ROUNDUP(size,0x100)>>8];
 	return heapNum1[size];
 }
@@ -561,56 +593,65 @@ static inline int getHeapSize(size_t size)
 /*
  * GLOBAL
  * 2Ѿ襵Ȥ˥ڡ꡼ʬ䤹
- * ϥڡ
  * parameters : Size by byte
  * return : allocate address or NULL
  */
 void *kmalloc(size_t size)
 {
-	char *addr;
+	union{
+		MEM_HEAD *head;
+		uint ui;
+	}mem,alloc;
 	int i;
-	HEAP_MEM *mem;
 
 
-	if(size==0)return NULL;
-	else if(size<=PAGE_SIZE)
+	if (size == 0)
+		return NULL;
+
+	if (size <= PAGE_SIZE)
 	{
-		addr=NULL;
-		i=getHeapSize(size);
+		i = getHeapSize(size);
 
-		enter_spinlock(&heap[i].gate);
+		enter_spinlock(&heapHead[i].gate);
 		{
-			mem=heap[i].mem;
-
-			if(mem==NULL)
+			if (heapHead[i].memTop == NULL)
 			{
 				/* 1ڡ꡼ݡ */
-				if((mem=allocKernelPage())==NULL)goto OUT;
+				mem.head = allocKernelPage();
+				if (mem.head == NULL)
+				{
+					exit_spinlock(&heapHead[i].gate);
+					return NULL;
+				}
 
 				/*
 				 * Ȥaddress狼褦ˤ롣
 				 */
-				getPhysTableFromAddr(mem)->heap_num=i;
+				getPhysTableFromAddr(mem.head)->heap_num = i;
 
-				heap[i].mem=mem;
-				mem->next=NULL;
-				mem->last=(char*)mem+PAGE_SIZE;
+				mem.head->next = NULL;
+				mem.head->size = PAGE_SIZE;
+				heapHead[i].memTop = mem.head;
 			}
-			addr=mem->last-heap[i].size;
-			if(addr==(char*)mem)heap[i].mem=mem->next;
-			else mem->last=addr;
+
+			mem.head = heapHead[i].memTop;
+			mem.head->size -= heapHead[i].size;
+			if (mem.head->size == 0)
+				heapHead[i].memTop = mem.head->next;
 		}
-OUT:
-		exit_spinlock(&heap[i].gate);
+		exit_spinlock(&heapHead[i].gate);
+
+		alloc.ui = mem.ui + mem.head->size;
 	}
 	else
 	{
-		size=ROUNDUP(size,PAGE_SIZE);
-		if((addr=allocMultiKernelPage(size/PAGE_SIZE))==NULL)return NULL;
-		getPhysTableFromAddr(addr)->heap_num=HEAP_NUM()+size/PAGE_SIZE;
+		size = ROUNDUP(size,PAGE_SIZE);
+		if ((alloc.head = allocMultiKernelPage(size / PAGE_SIZE)) == NULL)
+			return NULL;
+		getPhysTableFromAddr(alloc.head)->heap_num = HEAP_NUM() + size / PAGE_SIZE;
 	}
 
-	return addr;
+	return alloc.head;
 }
 
 
@@ -621,88 +662,104 @@ OUT:
  */
 void kfree(void *ptr)
 {
-	int i;
-	HEAP_MEM *mem,*prev;
-	uint m;
+	union{
+		MEM_HEAD *head;
+		uint ui;
+	}mem,free,prev,next;
+	uint i,m;
 
 
-	if(ptr==NULL)return;
+	if (ptr == NULL)
+		return;
 
-	if((i=getPhysTableFromAddr(ptr)->heap_num)==0)
+	if ((i = getPhysTableFromAddr(ptr)->heap_num) == 0)
+	{
 #ifdef DEBUG
-	{
-		printk("kfree() error! free address=%x\n",ptr);
-		return;
-	}
-	if((uint)ptr&(heap[i].size-1))
-	{
-		printk("kfree() error! illegal address=%x\n",ptr);
-		return;
-	}
-#else
-		return;
+		printk("kfree() : 0x%x did not allocated\n",ptr);
 #endif
-	if(i<HEAP_NUM())
-	{
-		mem=(HEAP_MEM*)ptr;
-		prev=(HEAP_MEM*)&heap[i];
+		return;
+	}
 
-		enter_spinlock(&heap[i].gate);
+	if (i < HEAP_NUM())
+	{
+		free.head = ptr;
+#ifdef DEBUG
+		/* Υå */
+		mem.head = free.head;
+		mem.ui &= heapHead[i].size - 1;
+		if (mem.ui != 0)
 		{
-			/* Link */
-			for(;prev->next!=NULL;prev=prev->next)
-				if(prev->next>=mem)break;
+			printk("kfree() : 0x%x is no complete\n",free.head);
+			return;
+		}	
+#endif
+		enter_spinlock(&heapHead[i].gate);
+		{
+			/* ꡼³󥯰֤򸡺 */
+			mem.head = (MEM_HEAD*)&heapHead[i];
+			for (;mem.head->next != NULL; mem.head = mem.head->next)
+				if (free.head < mem.head->next)
+					break;
+			prev = mem;
 #ifdef DEBUG
-			if(prev->next==mem)
+			if ((mem.head != (MEM_HEAD*)&heapHead[i]) && (free.ui < (mem.ui + mem.head->size)) )
 			{
-				printk("kfree() error! 0x%x memory is already freed\n",ptr);
-				goto NEXT;
+				printk("kfree() : 0x%x is already free\n",free.head);
+				goto EXIT;
 			}
 #endif
-			mem->next=prev->next;
-			prev->next=mem;
-			mem->last=(char*)ptr+heap[i].size;
-
-			/* ɥ쥹³ƤХ꡼Ĥʤ */
-			if(mem->last==(char*)mem->next)
+			/* Υ꡼ȷҤ */
+			next.head = prev.head->next;
+			if ((next.ui == (free.ui + heapHead[i].size)) && ((next.ui / PAGE_SIZE) == (free.ui / PAGE_SIZE)) )
 			{
-				mem->last=mem->next->last;
-				mem->next=mem->next->next;
-
-				/* Ĥʤ꡼ڡˤʤäڡ */
-				if(!((uint)mem&(PAGE_SIZE-1))&&!((uint)mem->last&(PAGE_SIZE-1)))
-				{
-					prev->next=mem->next;
-					getPhysTableFromAddr(ptr)->heap_num=0;
-					unlinkPhysTable((uint)mem);
-					goto NEXT;
-				}
+				free.head->next = next.head->next;
+				free.head->size = next.head->size + heapHead[i].size;
 			}
-			if(prev->last==(char*)mem)
+			else
 			{
-				prev->last=mem->last;
-				prev->next=mem->next;
+				free.head->next = next.head;
+				free.head->size = heapHead[i].size;
+			}
 
-				if(!((uint)prev&(PAGE_SIZE-1))&&!((uint)prev->last&(PAGE_SIZE-1)))
-				{
-					mem=prev;
-					prev=(HEAP_MEM*)&heap[i];
-					for(;prev->next!=mem;prev=prev->next);
-					prev->next=mem->next;
-					unlinkPhysTable((uint)mem);
-				}
+			/* Ĥʤ꡼ڡˤʤäڡ */
+			if (free.head->size == PAGE_SIZE)
+			{
+				prev.head->next = free.head->next;
+				getPhysTableFromAddr(free.head)->heap_num = 0;
+				unlinkPhysTable(free.ui);
+				goto EXIT;
+			}
+
+			/* Υ꡼ȷҤ */
+			if (((prev.ui + prev.head->size) == free.ui) && ((prev.ui / PAGE_SIZE) == (free.ui / PAGE_SIZE)) )
+			{
+				prev.head->next = free.head->next;
+				prev.head->size += free.head->size;
+				
+			}
+			else
+				prev.head->next = free.head;
+
+			/* Ĥʤ꡼ڡˤʤäڡ */
+			if (prev.head->size == PAGE_SIZE)
+			{
+				for (mem.head = (MEM_HEAD*)&heapHead[i]; mem.head->next != prev.head; mem.head = mem.head->next);
+				mem.head->next = free.head->next;
+				getPhysTableFromAddr(prev.head)->heap_num = 0;
+				unlinkPhysTable(prev.ui);
+				goto EXIT;
 			}
 		}
-NEXT:	exit_spinlock(&heap[i].gate);
+EXIT:	exit_spinlock(&heapHead[i].gate);
 	}
 	else
 	{
-		getPhysTableFromAddr(ptr)->heap_num=0;
-		m=(uint)ptr;
-		for(i-=HEAP_NUM();i>0;--i)
+		getPhysTableFromAddr(ptr)->heap_num = 0;
+		m = (uint)ptr;
+		for (i -= HEAP_NUM(); i > 0; --i)
 		{
 			unlinkPhysTable(m);
-			m+=PAGE_SIZE;
+			m += PAGE_SIZE;
 		}
 	}
 }
@@ -742,7 +799,7 @@ enum{
  * PUBLIC
  * ɥ쥹ڡǥ쥯ȥꥨȥ꡼ɥ쥹롣
  * parameters : ɥ쥹,ڡǥ쥯ȥꥢɥ쥹
- * return : ڡǥ쥯ȥꥢɥ쥹
+ * return : ڡǥ쥯ȥꥨȥ꡼ɥ쥹
  */
 static inline uint *getPageDirEntryAddr(uint addr,uint *page_dir)
 {
@@ -754,7 +811,7 @@ static inline uint *getPageDirEntryAddr(uint addr,uint *page_dir)
  * PUBLIC
  * ɥ쥹ڡơ֥륨ȥ꡼ɥ쥹롣
  * parameters : ɥ쥹,ڡơ֥륢ɥ쥹
- * return : ڡɥ쥹
+ * return : ڡơ֥륨ȥ꡼ɥ쥹
  */
 static inline uint *getPageEntryAddr(uint addr,uint *ptbl)
 {
@@ -762,6 +819,22 @@ static inline uint *getPageEntryAddr(uint addr,uint *ptbl)
 }
 
 
+/*
+ * PUBLIC
+ * ɥ쥹ڡơ֥륨ȥ꡼ɥ쥹롣
+ * parameters : ɥ쥹,ڡǥ쥯ȥꥢɥ쥹
+ * return : ڡơ֥륨ȥ꡼ɥ쥹
+ */
+/*static uint *getPageEntryAddrFromLinearAddr(uint addr,uint *page_dir)
+{
+	uint *ptbl;
+
+
+	ptbl=(uint*)(*getPageDirEntryAddr(addr,page_dir)&~(PAGE_SIZE-1));
+	return getPageEntryAddr(addr,ptbl);
+}
+*/
+
 /*
  * PUBLIC
  * flash page directory
@@ -900,32 +973,42 @@ static inline void setBlockEmpty(uint pentry)
 
 
 /*
+ * machine depend
  * PUBLIC
  * ڡХååץǥХλȲ1䤹
  * parameters : page entry
  */
 static inline void incBackupCount(uint *pentry)
 {
-	rotateR(pentry,24);
-	++(uchar)*pentry;
-	rotateL(pentry,24);
+	union{
+		uint ui;
+		uchar uc[4];
+	}value;
+
+
+	value.ui = *pentry;
+	++value.uc[3];
+	*pentry = value.ui;
 }
 
 
 /*
+ * machine depend
  * PUBLIC
  * ڡХååץǥХλȲ1餹
  * parameters : page entry
  */
 static inline void decBackupCount(uint *pentry)
 {
-	rotateR(pentry,24);
-	if((uchar)(--*pentry)==0)
-	{
-		rotateL(pentry,24);
+	union{
+		uint ui;
+		uchar uc[4];
+	}value;
+
+
+	value.ui = *pentry;
+	if (value.uc[3] == 0)
 		setBlockEmpty(*pentry);
-	}
-	else rotateL(pentry,24);
 }
 
 
@@ -943,21 +1026,23 @@ static int backupPage(uint phaddr,PROC *proc,uint *pentry)
 	enter_spinlock(&((MM_STRUCT*)proc->mm_struct)->pageDirGate);
 	{
 		/* Ǥ˥꡼ϳƤ롣 */
-		if(((*pentry&PAGE_PRESEN)==0)||(phaddr!=(*pentry&~(PAGE_SIZE-1))))
+		if (((*pentry & PAGE_PRESEN) == 0) || (phaddr != (*pentry & ~(PAGE_SIZE - 1))) )
 			goto EXIT;
 
 		/* page tableԺߤ */
-		*pentry&=~PAGE_PRESEN;
+		*pentry &= ~PAGE_PRESEN;
 
 		/* ꤵץ¹ʤpage tableTBLեå夹 */
 		if((proc->cpu!=get_current_cpu())&&(cputask[proc->cpu].current_task==proc))
 			issue_interrupt_to_cpu(proc->cpu,FLASH_PAGEDIR_VECT);
 
 		/* Store page. */
-		if((blk=getEmptyBackup())==-1)goto ERR;
-		if(phaddr<KERNEL_END)
+		if ((blk = getEmptyBackup()) == -1)
+			goto ERR;
+		if (phaddr < KERNEL_END)
 		{
-			if(write_cache(pbDin,(void*)phaddr,pbSectors,blk)!=pbSectors)goto ERR;
+			if (write_cache(pbDin,(void*)phaddr,pbSectors,blk) != pbSectors)
+				goto ERR;
 		}
 		else
 		{
@@ -1007,7 +1092,8 @@ ERR:
 static int restorePage(void *buf,uint *pentry)
 {
 	/* ǥХɤ߹ߡ */
-	if(read_cache(pbDin,buf,pbSectors,getBlock(*pentry))!=pbSectors)return -1;
+	if (read_cache(pbDin,buf,pbSectors,getBlock(*pentry)) != pbSectors)
+		return -1;
 
 	/* ȥ󥿤򸺤餹 */
 	decBackupCount(pentry);
@@ -1028,10 +1114,11 @@ int initBackupDev(const char *dev_path)
 
 
 	/* Device open. */
-	if((pbDin=mountPageBackupDev(dev_path))<0)return pbDin;
+	if ((pbDin = mountPageBackupDev(dev_path)) < 0)
+		return pbDin;
 
 	/* ѡƥ󥿥פγǧ */
-	if(get_dev_stat(pbDin,&dev_stat)==-1)return -1;
+	if(get_devstat(pbDin,&dev_stat)==-1)return -1;
 	if(dev_stat.prt_type!=PT_TYPE_PAGING)return -1;
 	pbSectors=PAGE_SIZE/dev_stat.sect_size;
 
@@ -1072,7 +1159,7 @@ int initBackupDev(const char *dev_path)
  * ȥ꡼ɬ¸ߤƤΤȤ롣
  * parameters : page entry address
  */
-static inline void setCow(uint *pentry)
+static inline void setCopyOnWrite(uint *pentry)
 {
 	if((*pentry&PAGE_PRESEN)==0)incBackupCount(pentry);		/* Хåå */
 	else
@@ -1177,7 +1264,7 @@ static inline int cpystr(char *s1,const char *s2)
 void *forkPage(MM_STRUCT *org_mm,void **stack)
 {
 	void *user_stack;
-	uint *pdir,*ptbl,*org_ptbl;
+	uint *pdir,*ptbl,*org_pdir,*org_ptbl;
 	MM_STRUCT *mmstruct;
 	int i,j;
 
@@ -1187,8 +1274,9 @@ void *forkPage(MM_STRUCT *org_mm,void **stack)
 	mmstruct->pageDirGate=0;
 
 	/* ƤΥڡǥ쥯ȥҤ˥ԡ롣 */
+	org_pdir=org_mm->pageDir;
 	if((pdir=(uint*)allocKernelPage())==NULL)return NULL;
-	memcpy(pdir,org_mm->pageDir,PAGE_SIZE);
+	memcpy(pdir,org_pdir,PAGE_SIZE);
 	mmstruct->pageDir=pdir;
 
 	/* ͥ륹åƤ롣 */
@@ -1214,17 +1302,19 @@ void *forkPage(MM_STRUCT *org_mm,void **stack)
 			for(j=0;j<PAGE_SIZE/sizeof(uint);++j)
 			{
 				if(org_ptbl[j]==0)break;
-				setCow(&org_ptbl[j]);
+				setCopyOnWrite(&org_ptbl[j]);
 			}
 
+			/* ǥ쥯ȥꥨȥ꡼˥ԡ饤ȥե饰ΩƤ롣 */
 			pdir[i]|=PT_COPY_ON_WRITE;
-			linkPhysTable((uint)org_ptbl);
+			linkPhysTable(pdir[i]&~(PAGE_SIZE-1));
+			org_pdir[i]|=PT_COPY_ON_WRITE;
 		}
 		mmstruct->lastPageAddr=(i-1)*PAGE_DIR_SIZE+j*PAGE_SIZE-1;
 
 		/* 桼åγơ */
 		if((ptbl=(uint*)allocKernelPage())==NULL)goto ERR;
-		if((user_stack=allocUserPage(&ptbl[PAGE_SIZE/sizeof(uint)-1]))==NULL)goto ERR2;
+		if((user_stack = allocUserPage(&ptbl[PAGE_SIZE/sizeof(uint)-1]))==NULL)goto ERR2;
 		if(pdir[USER_STACK_BEG/PAGE_DIR_SIZE]!=0)
 		{
 			org_ptbl=(uint*)(pdir[USER_STACK_BEG/PAGE_DIR_SIZE]&~(PAGE_SIZE-1));
@@ -1235,7 +1325,7 @@ void *forkPage(MM_STRUCT *org_mm,void **stack)
 			{
 				while(i>=0)
 				{
-					setCow(&org_ptbl[i]);
+					setCopyOnWrite(&org_ptbl[i]);
 					if(org_ptbl[--i]==0)break;
 				}
 			}
@@ -1251,7 +1341,6 @@ void *forkPage(MM_STRUCT *org_mm,void **stack)
 	exit_spinlock(&org_mm->pageDirGate);
 
 	return mmstruct;
-
 ERR2:
 	unlinkPhysTable((uint)ptbl);
 ERR:
@@ -1299,7 +1388,7 @@ int loadBinary(OPEN_F *open_f,size_t size,size_t load_size,uint address,int attr
 
 			/* ꡼°ꤹ롣 */
 			pentry=getPageEntryAddr(addr,ptbl);
-			if((page=allocUserPage(pentry))==NULL)goto ERR;
+			if ((page = allocUserPage(pentry))==NULL)goto ERR;
 			*pentry=(uint)page|PAGE_PRESEN|PAGE_USER|PAGE_RW;
 		}
 	}
@@ -1442,6 +1531,21 @@ void releasePageDir(void *mm_struct)
  *  桼åڡԺߤνθľ(¤ߤ)
  * parameters : page fault address,Exception frame
  */
+static void printErr(uint errnum,uint addr,uint eip,uint cs,uint esp,uint ss)
+{
+	if(cs==KERNEL_CODE_DES)
+	{
+#ifdef DEBUG	
+		printSyscallEntry(5);
+#endif		
+		printk("error number=0x%x fault address=0x%x eip=0x%x cs=0x%x process=%x\n",
+			errnum,addr,eip,cs,get_current_task());
+	}		
+	else
+		printk("error number=0x%x fault address=0x%x eip=0x%x cs=0x%x esp=0x%x,ss=0x%x\n",
+			errnum,addr,eip,cs,esp,ss);
+}
+
 void pageFault(uint fault_addr,EXCEPT_FRAME frame)
 {
 	/* 顼ɥӥåȥޥ */
@@ -1460,17 +1564,26 @@ void pageFault(uint fault_addr,EXCEPT_FRAME frame)
 	MM_STRUCT *mm_struct;
 
 
-	mm_struct=(MM_STRUCT*)get_current_task()->mm_struct;
+	mm_struct = get_current_task()->mm_struct;
 
 	/* øڡȿ */
-	if((fault_addr<USER_BEG)||(fault_addr>=IOMAP_BEG)||((fault_addr>mm_struct->lastPageAddr)&&(fault_addr<USER_STACK_BEG)))
+	if ((fault_addr < USER_BEG)   ||
+	    (IOMAP_BEG <= fault_addr) ||
+	    ((mm_struct->lastPageAddr < fault_addr) && (fault_addr < USER_STACK_BEG)) )
 	{
+		/* 桼ʥϥɥ餫ξ硣 */
+		if((fault_addr==(uint)returnUserHandler)&&(getBeforeEsp()))
+		{
+			get_current_task()->nest_count-=1;
+			returnUserHandler();
+		}	
+
 		printk("\nPage privilege violation!\n");
-		printk("error number=0x%x fault address=0x%x eip=0x%x cs=0x%x esp=0x%x,ss=0x%x,proc=%x\n",
-				frame.error,fault_addr,frame.eip,frame.cs,frame.user_esp,frame.user_ss,get_current_task());
-
+		printErr(frame.error,fault_addr,frame.eip,frame.cs,frame.user_esp,frame.user_ss);
 		forceSendSignal(get_current_task(),SIGSEGV);
-
+/***************************************/
+idle();
+/***************************************/
 		return;
 	}
 
@@ -1482,18 +1595,18 @@ void pageFault(uint fault_addr,EXCEPT_FRAME frame)
 		/* ڡǥ쥯ȥγơ */
 		if(*dentry==0)
 		{
-			if((*dentry=(uint)allocKernelPage())==0)
+			if ((*dentry = (uint)allocKernelPage()) == 0)
 			{
 				printk("Fail alloc physical page!\n");
 				sys_exit(1);
 			}
-			memset((void*)*dentry,0,PAGE_SIZE);
-			*dentry|=PAGE_PRESEN|PAGE_RW|PAGE_USER;
+			memset ((void*)*dentry,0,PAGE_SIZE);
+			*dentry |= PAGE_PRESEN | PAGE_RW | PAGE_USER;
 		}
 
 		/* ʪڡγơ */
-		current_pentry=getPageEntryAddr(fault_addr,(uint*)(*dentry&~(PAGE_SIZE-1)));
-		if((alloc_page=allocUserPage(current_pentry))==NULL)
+		current_pentry = getPageEntryAddr(fault_addr,(uint*)(*dentry&~(PAGE_SIZE-1)));
+		if ((alloc_page = allocUserPage(current_pentry)) == NULL)
 		{
 			printk("Fail alloc physical page!\n ");
 			sys_exit(1);
@@ -1502,8 +1615,8 @@ void pageFault(uint fault_addr,EXCEPT_FRAME frame)
 		enter_spinlock(&mm_struct->pageDirGate);
 		{
 			/* Хååפꡣ */
-			if(*current_pentry!=0)
-				if(restorePage(alloc_page,current_pentry)==-1)
+			if (*current_pentry != 0)
+				if (restorePage(alloc_page,current_pentry) == -1)
 				{
 					printk("Fail restore page!\n ");
 					exit_spinlock(&mm_struct->pageDirGate);
@@ -1512,59 +1625,63 @@ void pageFault(uint fault_addr,EXCEPT_FRAME frame)
 					return;
 				}
 
-			*current_pentry=(uint)alloc_page|PAGE_PRESEN|PAGE_RW|PAGE_USER;
+			*current_pentry = (uint)alloc_page | PAGE_PRESEN | PAGE_RW | PAGE_USER;
 		}
 		exit_spinlock(&mm_struct->pageDirGate);
 	}
 	else	/* ڡȿ */
 	{
-		/* 桼ʥϥɥ餫ξ硣 */
-		if((fault_addr==(uint)returnUserHandler)&&(getBeforeEsp()))returnUserHandler();
-
 		current_pentry=getPageEntryAddr(fault_addr,(uint*)(*dentry&~(PAGE_SIZE-1)));
 
+		/* ԡ饤Ƚ */
 		if(*current_pentry&PT_COPY_ON_WRITE)
 		{
-			/*
-			 * ԡ饤Ƚ
-			 * ڡǥ쥯ȥꥨȥ꡼˥ԡ饤ȥե饰ꤵƤϡ
-			 * ڡơ֥Ƥ롣
-			 */
-			if(pdir[fault_addr/PAGE_DIR_SIZE]&PT_COPY_ON_WRITE)
+			/* ʪ꡼ȥȤǧ */
+			if(delPhysRefCount(*current_pentry)==0)
 			{
-				array=fault_addr/PAGE_DIR_SIZE;
+				uint *new_entry=current_pentry;
 
-				if((alloc_page=allocKernelPage())==NULL)
+				/*
+				 * ڡǥ쥯ȥꥨȥ꡼˥ԡ饤ȥե饰ꤵƤϡ
+				 * ڡơ֥Ƥ롣
+				 */
+				if(pdir[fault_addr/PAGE_DIR_SIZE]&PT_COPY_ON_WRITE)
 				{
-					printk("Fail alloc physical page!\n ");
+					void *alloc_tbl;
+
+					array=fault_addr/PAGE_DIR_SIZE;
+
+					/* ʪ꡼ȥȤǧ */
+					if(delPhysRefCount(pdir[array])==0)
+					{
+						if((alloc_tbl=allocKernelPage())==NULL)
+						{
+							printk("Fail alloc physical page!\n ");
+							sys_exit(1);
+						}
+
+						memcpy(alloc_tbl,(void*)(pdir[array]&~(PAGE_SIZE-1)),PAGE_SIZE);
+						pdir[array]=((pdir[array]&(PAGE_SIZE-1))|(uint)alloc_tbl)&~PT_COPY_ON_WRITE;
+						new_entry=(uint*)(((uint)current_pentry&(PAGE_SIZE-1))|(uint)alloc_tbl);
+					}
+					else pdir[array]&=~PT_COPY_ON_WRITE;
+				}
+
+				/* ʪڡơڡƤ򥳥ԡ롣 */
+				if((alloc_page=allocUserPage(new_entry))==NULL)
+				{
+					printk("Fail alloc physical page!\n");
 					sys_exit(1);
 				}
-				memcpy(alloc_page,(void*)(pdir[array]&~(PAGE_SIZE-1)),PAGE_SIZE);
-				pdir[array]=((pdir[array]&(PAGE_SIZE-1))|(uint)alloc_page)&~PT_COPY_ON_WRITE;
+				memcpy(alloc_page,(void*)(*current_pentry&~(PAGE_SIZE-1)),PAGE_SIZE);
+				*new_entry=(uint)alloc_page|PAGE_PRESEN|PAGE_RW|PAGE_USER;
 			}
-
-			/* ʪڡγơ */
-			if((alloc_page=allocUserPage(current_pentry))==NULL)
-			{
-				printk("Fail alloc physical page!\n ");
-				sys_exit(1);
-			}
-
-			/* ڡƤƤڡ˥ԡ롣 */
-			memcpy(alloc_page,(void*)(*current_pentry&~(PAGE_SIZE-1)),PAGE_SIZE);
-
-			*current_pentry=(uint)alloc_page|PAGE_PRESEN|PAGE_RW|PAGE_USER;
+			else *current_pentry=(*current_pentry&~PT_COPY_ON_WRITE)|PAGE_RW;
 		}
 		else	/* ɤ߹ߥڡȿ */
 		{
 			printk("\nRead only page violation!\n");
-			if(frame.cs==KERNEL_CODE_DES)
-				printk("error number=0x%x fault address=0x%x eip=0x%x cs=0x%x process=%x\n",
-					frame.error,fault_addr,frame.eip,frame.cs,get_current_task());
-			else
-				printk("error number=0x%x fault address=0x%x eip=0x%x cs=0x%x esp=0x%x,ss=0x%x\n",
-					frame.error,fault_addr,frame.eip,frame.cs,frame.user_esp,frame.user_ss);
-
+			printErr(frame.error,fault_addr,frame.eip,frame.cs,frame.user_esp,frame.user_ss);
 			forceSendSignal(get_current_task(),SIGSEGV);
 
 			return;
@@ -1739,13 +1856,10 @@ void initMm()
 	initPhysMemTable();
 }
 /************************************************************************/
-void test_mm(void *mm)
+int test_mm(uint *page)
 {
-	uint *pdir=(uint*)0x172000;
-	uint *old=(uint*)0x91000;
-	int i;
+	printk("40000c39=%x 40000080=%x\n",*(uint*)((uint)page + 0xc39),*(uint*)((uint)page + 0x80));
 
-	for(i=0;i<PAGE_SIZE/4;++i)
-		if(pdir[i]!=old[i])printk("%x-%x-%x,",i*0x400000,pdir[i],old[i]);
+	return 0;
 }
 /*************************************************************************/
