@@ -15,7 +15,7 @@
 #include<types.h>
 #include<proc.h>
 #include<fs.h>
-#include<share/system.h>
+#include<sys/system.h>
 
 
 #define PAGE_SIZE 0x1000	/* Page size */
