@@ -8,7 +8,7 @@
  */
 
 
-#include<user/include/share/user_config.h>
+#include<sys/user_config.h>
 #include<types.h>
 #include<lib.h>
 #include<lock.h>
