@@ -20,7 +20,17 @@
 #include<net/udp.h>
 #include<net/tcp.h>
 #include<net/net.h>
-#include<share/socket.h>
+#include<net/ether.h>
+
+
+enum{
+	IP_PROTNUM_MAX=17,		/* IP ̥ץȥʥС͡ */
+	SOCKET_MAX=10,			/* ץλĥåȤκ */
+};
+
+
+static FS *fs;
+static SOCKET_INFO *sockInfo[IP_PROTNUM_MAX+1];
 
 
 /************************************************************************************
@@ -112,6 +122,30 @@ ushort calcSumM(uint *data1,uint *data2,int size1,int size2)
 }
 
 
+/*
+ * GLOBAL
+ * Get  ephemeral port number.
+ * return : ephemeral port number
+ */
+ushort getEphemeralPort()
+{
+	enum{BEGIN_EPHM_PORT=1024};		/* եݡȤκǽֹ档 */
+
+	static int gate=0;
+	static int volatile port=BEGIN_EPHM_PORT;
+	int pt;
+
+
+	enter_spinlock(&gate);
+	{
+		pt=port++;
+	}
+	exit_spinlock(&gate);
+
+	return pt;
+}
+
+
 /*
  * GLOBAL
  * Close socket.
@@ -121,9 +155,51 @@ ushort calcSumM(uint *data1,uint *data2,int size1,int size2)
 void closeSocket(SOCKET *sc)
 {
 	/* åȤγ */
-	if(sc->sockType==SOCK_STREAM)
-		releaseSckFromConect(sc);
-	else kfree(sc);
+	switch (sc->sockType)
+	{
+		case SOCK_STREAM:
+			if (releaseSckFromConect(sc) != 0)
+				return;
+			break;
+		case SOCK_DGRAM:
+			releaseSockUdp(sc);
+			break;
+	}
+	
+	kfree(sc);
+}
+
+
+/*
+ * GLOBAL
+ * Make socket
+ * parameters : sockest type,protocol,socket address
+ * return : 0 or error number
+ */
+int makeSocket(int type, int protocol,SOCKET **psc)
+{
+	SOCKET *sc;
+	
+	
+	if ((sc = kmalloc(sizeof(SOCKET))) == NULL)
+		return -ENOMEM;
+	sc->fs = fs;
+	sc->sc = sc;
+	sc->sockType = type;
+	sc->stat = 0;
+	sc->sinf = sockInfo[protocol];
+	sc->recvBuf = NULL;
+	sc->waitProc = NULL;
+	sc->srcip = 0;
+	sc->dstip = 0;
+	sc->srcport = 0;
+	sc->connect = NULL;
+	sc->lockGate = 0;
+	sc->newsc_next = NULL;
+
+	*psc = sc;
+
+	return 0;
 }
 
 
@@ -134,16 +210,6 @@ void closeSocket(SOCKET *sc)
  ************************************************************************************/
 
 
-enum{
-	IP_PROTNUM_MAX=17,		/* IP ̥ץȥʥС͡ */
-	SOCKET_MAX=10,			/* ץλĥåȤκ */
-};
-
-
-static SOCKET_INFO *sockInfo[IP_PROTNUM_MAX+1];
-static FS *fs;
-
-
 int sys_socket(int domain, int type, int protocol)
 {
 	int num;
@@ -154,23 +220,28 @@ int sys_socket(int domain, int type, int protocol)
 
 	/* ե륪ץå */
 	if((rest=checkFileOpen())<0)return rest;
-
 	switch(domain)
 	{
 		case AF_INET:
 			switch(type)
 			{
 				case SOCK_DGRAM:
-					if((protocol==0)||(protocol==IPPROTO_UDP))num=IPPROTO_UDP;
-					else return -EPROTONOSUPPORT;
+					if((protocol == 0)||(protocol == IPPROTO_UDP))
+						num = IPPROTO_UDP;
+					else
+						return -EPROTONOSUPPORT;
 					break;
 				case SOCK_STREAM:
-					if((protocol==0)||(protocol==IPPROTO_TCP))num=IPPROTO_TCP;
-					else return -EPROTONOSUPPORT;
+					if((protocol == 0)||(protocol == IPPROTO_TCP))
+						num = IPPROTO_TCP;
+					else 
+						return -EPROTONOSUPPORT;
 					break;
 				case SOCK_RAW:
-					if((protocol==0)||(protocol==IPPROTO_ICMP))num=IPPROTO_ICMP;
-					else return -EPROTONOSUPPORT;
+					if((protocol == 0)||(protocol == IPPROTO_ICMP))
+						num = IPPROTO_ICMP;
+					else 
+						return -EPROTONOSUPPORT;
 					break;
 				default:
 					return -EPROTONOSUPPORT;
@@ -180,20 +251,15 @@ int sys_socket(int domain, int type, int protocol)
 			return -EPROTONOSUPPORT;
 	}
 
-	if((sc=kmalloc(sizeof(SOCKET)))==NULL)return -ENOMEM;
-	sc->fs=fs;
-	sc->sc=sc;
-	if((fd=setFd((VENTRY*)&sc->fs,FD_FORM_NETSOKET,O_RDWR,0,(F_DSC*)sc))<0)
+	rest = makeSocket(type,num,&sc);
+	if (rest < 0)
+		return rest;
+	fd = setFd((VENTRY*)&sc->fs,FD_FORM_NETSOKET,O_RDWR,0,(F_DSC*)sc);
+	if(fd < 0)
 	{
 		kfree(sc);
 		return fd;
 	}
-	sc->sockType=type;
-	sc->stat=0;
-	sc->update=0;
-	sc->sinf=sockInfo[num];
-	sc->buf=NULL;
-	sc->wait.flag=0;
 
 	/* ץϿ */
 	return fd;
@@ -208,15 +274,24 @@ int sys_send(int sd,const void *msg,size_t len,int flags,struct sockaddr *to)
 	if((sc=(SOCKET*)getFd(sd))==NULL)return -EBADF;
 	if(sc->fd.form!=FD_FORM_NETSOKET)return -ENOTSOCK;
 	if((to!=NULL)&&(checkMem(to)==-1))return -EFAULT;
+	if (sc->srcport == 0)
+		sc->srcport = swapWord(getEphemeralPort());
 
 	return ((SOCKET_INFO*)sc->sinf)->send(sc,msg,len,flags,to);
 }
 
 
+int sys_sendmsg(int s, const struct msghdr *msg, int flags)
+{
+	if(checkMem((void*)msg)==-1)return -EFAULT;
+
+	return -ENOTSUP;
+}
+
+
 int sys_recv(int sd,void *buf,size_t len,int flags,struct sockaddr *from,socklen_t *fromlen)
 {
 	int rest;
-	uint32_t srcip;
 	SOCKET *sc;
 
 
@@ -226,17 +301,27 @@ int sys_recv(int sd,void *buf,size_t len,int flags,struct sockaddr *from,socklen
 	if(from!=NULL)
 	{
 		if((checkMem(from)==-1)||(checkMem(fromlen)==-1))return -EFAULT;
-		rest=((SOCKET_INFO*)sc->sinf)->recv(sc,buf,len,flags,&srcip);
-		if(*fromlen>=sizeof(struct sockaddr)+sizeof(uint32_t))
+		rest=((SOCKET_INFO*)sc->sinf)->recv(sc,buf,len,flags);
+		if(*fromlen>=sizeof(struct sockaddr_in))
 		{
-			*(uint32_t*)from->sa_data=srcip;
-			*fromlen=sizeof(struct sockaddr)+sizeof(uint32_t);
+			((struct sockaddr_in*)from)->sin_family=AF_INET;
+			((struct sockaddr_in*)from)->sin_addr.s_addr=sc->dstip;
+			((struct sockaddr_in*)from)->sin_port=sc->dstport;
+			*fromlen=sizeof(struct sockaddr_in);
 		}
 
 		return rest;
 	}
 	else
-		return ((SOCKET_INFO*)sc->sinf)->recv(sc,buf,len,flags,&srcip);
+		return ((SOCKET_INFO*)sc->sinf)->recv(sc,buf,len,flags);
+}
+
+
+int sys_recvmsg(int s, struct msghdr *msg, int flags)
+{
+	if(checkMem(msg)==-1)return -EFAULT;
+
+	return -ENOTSUP;
 }
 
 
@@ -245,12 +330,18 @@ int sys_connect(int sockfd, const struct sockaddr *serv_addr)
 	SOCKET *sc;
 
 
-	if((sc=(SOCKET*)getFd(sockfd))==NULL)return -EBADF;
-	if(sc->fd.form!=FD_FORM_NETSOKET)return -ENOTSOCK;
-	if(checkMem((void*)serv_addr)==-1)return -EFAULT;
+	sc=(SOCKET*)getFd(sockfd);
+	if (sc == NULL)
+		return -EBADF;
+	if (sc->fd.form != FD_FORM_NETSOKET)
+		return -ENOTSOCK;
+	if(checkMem((void*)serv_addr) == -1)
+		return -EFAULT;
 
-	sc->dstip=((struct sockaddr_in*)serv_addr)->sin_addr.s_addr;
-	sc->dstport=((struct sockaddr_in*)serv_addr)->sin_port;
+	sc->dstip = ((struct sockaddr_in*)serv_addr)->sin_addr.s_addr;
+	sc->dstport = ((struct sockaddr_in*)serv_addr)->sin_port;
+	if (sc->srcport == 0)
+		sc->srcport = swapWord(getEphemeralPort());
 
 	return ((SOCKET_INFO*)sc->sinf)->connect(sc);
 }
@@ -273,12 +364,14 @@ int sys_bind(int sockfd, struct sockaddr *my_addr)
 	SOCKET *sc;
 
 
-	if((sc=(SOCKET*)getFd(sockfd))==NULL)return -EBADF;
-	if(sc->fd.form!=FD_FORM_NETSOKET)return -ENOTSOCK;
-	if(checkMem((void*)my_addr)==-1)return -EFAULT;
+	if ((sc = (SOCKET*)getFd(sockfd)) == NULL)return -EBADF;
+	if (sc->fd.form!=FD_FORM_NETSOKET)return -ENOTSOCK;
+	if (checkMem((void*)my_addr) == -1)return -EFAULT;
 
-	sc->srcip=((struct sockaddr_in*)my_addr)->sin_addr.s_addr;
-	sc->srcport=((struct sockaddr_in*)my_addr)->sin_port;
+	sc->srcip = ((struct sockaddr_in*)my_addr)->sin_addr.s_addr;
+	sc->srcport = ((struct sockaddr_in*)my_addr)->sin_port;
+	if (sc->srcport == 0)
+		sc->srcport = swapWord(getEphemeralPort());
 
 	return 0;
 }
@@ -289,10 +382,12 @@ int sys_listen(int s, int backlog)
 	SOCKET *sc;
 
 
-	if((sc=(SOCKET*)getFd(s))==NULL)return -EBADF;
-	if(sc->fd.form!=FD_FORM_NETSOKET)return -ENOTSOCK;
+	if((sc = (SOCKET*)getFd(s)) == NULL)
+		return -EBADF;
+	if(sc->fd.form != FD_FORM_NETSOKET)
+		return -ENOTSOCK;
 
-	return ((SOCKET_INFO*)sc->sinf)->listen(sc);
+	return ((SOCKET_INFO*)sc->sinf)->listen(sc,s);
 }
 
 
@@ -304,45 +399,90 @@ int sys_accept(int s, struct sockaddr *addr, socklen_t *addrlen)
 	uint32_t srcip;
 
 
-	if((sc=(SOCKET*)getFd(s))==NULL)return -EBADF;
-	if(sc->fd.form!=FD_FORM_NETSOKET)return -ENOTSOCK;
-	if((sc->sockType!=SOCK_STREAM)&&(sc->sockType!=SOCK_DGRAM))return -EOPNOTSUPP;
+	if ((sc = (SOCKET*)getFd(s)) == NULL)
+		return -EBADF;
+	if (sc->fd.form != FD_FORM_NETSOKET)
+		return -ENOTSOCK;
+	if ((sc->sockType != SOCK_STREAM) && (sc->sockType != SOCK_DGRAM))
+		return -EOPNOTSUPP;
 
-	if(addr!=NULL)
+	if (addr != NULL)
 	{
-		if((checkMem(addr)==-1)||(checkMem(addrlen)==-1))return -EFAULT;
+		if ((checkMem(addr) == -1) || (checkMem(addrlen) == -1))
+			return -EFAULT;
 	}
 
-	/* åȤγơ */
 	/* ե륪ץå */
-	if((rest=checkFileOpen())<0)return rest;
-	if((newsc=kmalloc(sizeof(SOCKET)))==NULL)return -ENOMEM;
-	newsc->fs=fs;
-	newsc->sc=newsc;
-	if((fd=setFd((VENTRY*)&sc->fs,FD_FORM_NETSOKET,O_RDWR,0,(F_DSC*)newsc))<0)
+	if ((rest = checkFileOpen()) < 0)
+		return rest;
+
+	/* ³׵Ԥ */
+	rest = ((SOCKET_INFO*)sc->sinf)->accept(sc,&newsc,&srcip);
+	if (rest < 0)
+		return rest;
+	fd = setFd((VENTRY*)&newsc->fs,FD_FORM_NETSOKET,O_RDWR,0,(F_DSC*)newsc);
+	if (fd < 0)
 	{
 		kfree(newsc);
 		return fd;
 	}
 
-	/* ³׵Ԥ */
-	if((rest=((SOCKET_INFO*)sc->sinf)->accept(sc,newsc,&srcip))<0)
-	{
-		sys_close(fd);
-		return rest;
-	}
-
-	if(addr!=NULL)
-		if(*addrlen>=sizeof(struct sockaddr)+sizeof(uint32_t))
+	if (addr != NULL)
+		if (*addrlen >= sizeof(struct sockaddr) + sizeof(uint32_t))
 		{
-			*(uint32_t*)addr->sa_data=srcip;
-			*addrlen=sizeof(struct sockaddr)+sizeof(uint32_t);
+			*(uint32_t*)addr->sa_data = srcip;
+			*addrlen = sizeof(struct sockaddr) + sizeof(uint32_t);
 		}
 
 	return fd;
 }
 
 
+/*
+ * IPV4Τб
+ * parameters : file descriptor,flag,address buffer
+ */
+int sys_getsockaddr(int fd,int flag,in_addr_t *addr)
+{
+	SOCKET *sc;
+
+
+	if((sc=(SOCKET*)getFd(fd))==NULL)return -EBADF;
+	if(sc->fd.form!=FD_FORM_NETSOKET)return -ENOTSOCK;
+
+	if(flag==GET_SOCKADDR_REMOT)
+	{
+		if((sc->stat&SOCK_CNCT_ON)==0)return -ENOTCONN;
+		((struct sockaddr_in*)addr)->sin_addr.s_addr = sc->dstip;
+		((struct sockaddr_in*)addr)->sin_port = sc->dstport;
+	}
+	else
+	{
+		if (sc->srcip == 0)
+			((struct sockaddr_in*)addr)->sin_addr.s_addr = getSrcip(sc->dstip);
+		else	
+			((struct sockaddr_in*)addr)->sin_addr.s_addr = sc->srcip;
+		((struct sockaddr_in*)addr)->sin_port = sc->srcport;
+	}
+
+	return 0;
+}
+
+
+int sys_getsockopt(int fd, int level, int optname, void *optval,socklen_t *optlen)
+{
+	/* ̤ */
+	return -ENOPROTOOPT;
+}
+
+
+int sys_setsockopt(int fd, int level, int optname, void *optval,socklen_t *optlen)
+{
+	/* ̤ */
+	return -ENOPROTOOPT;
+}
+
+
 /************************************************************************************
  *
  * File system interface
@@ -365,17 +505,14 @@ static int open(const char *path,int din,uint dir_inode,int flag,uint *inode)
 	return -ENOTSUP;
 }
 
-static int close(int din,uint inode)
+static int close(int din,uint inode,int type)
 {
-	return -ENOTSUP;
+	return 0;
 }
 
 static int read(int din,uint inode,void *buf,size_t size,size_t begin)
 {
-	uint32_t srcip;
-
-
-	return ((SOCKET_INFO*)((SOCKET*)inode)->sinf)->recv((SOCKET*)inode,buf,size,0,&srcip);
+	return ((SOCKET_INFO*)((SOCKET*)inode)->sinf)->recv((SOCKET*)inode,buf,size,0);
 }
 
 static int write(int din,uint inode,void *buf,size_t size,size_t begin)
@@ -385,6 +522,12 @@ static int write(int din,uint inode,void *buf,size_t size,size_t begin)
 
 static int ioctl(int din,uint inode,int prm_num,int cmd,...)
 {
+	switch(cmd)
+	{
+		case SIOCATMARK:
+			return 0;
+	}
+
 	return -ENOTSUP;
 }
 
@@ -393,12 +536,12 @@ static int rename(int din,uint old_dinode,const char *old_path,uint new_dinode,c
 	return -ENOTSUP;
 }
 
-static int creat(const char *path,int din,uint dir_inode,int mode)
+static int creat(const char *path,int din,uint dir_inode,int mode,uint *inode)
 {
 	return -ENOTSUP;
 }
 
-static int opendir(const char *path,int din,uint dir_inode,DIR *dir)
+static int opendir(const char *path,int din,uint dir_inode,uint *inode,uint *block)
 {
 	return -ENOTSUP;
 }
@@ -408,7 +551,7 @@ static int mkdir(const char *path,int din,uint dir_inode,int mode)
 	return -ENOTSUP;
 }
 
-static int stat(int din,uint inode,FSTAT *fstat)
+static int stat(int din,uint inode,int type,FSTAT *fstat)
 {
 	return -ENOTSUP;
 }
@@ -418,12 +561,12 @@ static int delete(const char *path,int din,uint dir_inode,int type)
 	return -ENOTSUP;
 }
 
-static int readdir(int din,uint inode,DIR *dir,char *name)
+static int readdir(int din,uint inode,uint *block,uint *index,char *name)
 {
 	return -ENOTSUP;
 }
 
-static int chattr(int din,uint inode,uint mode,uint uid,uint gid,uint atime,uint mtime)
+static int chattr(int din,uint inode,int type,uint mode,uint uid,uint gid,uint atime,uint mtime)
 {
 	return -ENOTSUP;
 }
@@ -433,6 +576,17 @@ static int mklink(int din,uint oldinode,uint newind,const char *newpath)
 	return -ENOTSUP;
 }
 
+static int select(uint inode,int flag,int polling)
+{
+	return ((SOCKET_INFO*)((SOCKET*)inode)->sinf)->select((SOCKET*)inode,flag,polling);
+}
+
+
+static int fsstat(int din,struct statfs *buf)
+{
+	return -ENOTSUP;
+}
+
 
 /************************************************************************************
  *
@@ -443,7 +597,7 @@ static int mklink(int din,uint oldinode,uint newind,const char *newpath)
 
 static FS netfs={
 	"net",mount,umount,open,close,read,write,ioctl,rename,creat,opendir,mkdir,stat,delete,readdir,chattr,
-	mklink,NULL
+	mklink,select,fsstat,NULL
 };
 
 
