@@ -279,7 +279,7 @@ PROC *searchProcList(int pid)
  *
  *******************************************************************************************************/
 
-extern int active_cpu;		/* ư CPU */
+extern int *active_cpu;		/* ư CPU */
 
 /* PUBLIC */
 static int procLockGate=0;		/* ¾Υץ¤λȻΥåȡ */
@@ -299,7 +299,7 @@ static int GetFewTaskCpu()
 
 
 	num=cputask[0].proc_num;
-	for(i=1;i<active_cpu;++i)
+	for(i = 1; i < *active_cpu; ++i)
 		if(cputask[i].proc_num<num)
 		{
 			num=cputask[i].proc_num;
@@ -435,10 +435,10 @@ static void releaseChildProc(PROC *proc)
  * PUBLIC
  * 1 exitҥץ롣
  * 2 λҥץξ֤𤹤롣
- * parameters : process structure,search pid,search gid,task state,state address
+ * parameters : process structure,search pid,search pgid,task state,state address
  * return : pid or 0 or error number
  */
-static int searchExitState(PROC *proc,int pid,int gid,int state,int *stat_loc)
+static int searchExitState(PROC *proc,pid_t pid,pid_t pgid,int state,int *stat_loc)
 {
 	int rest=-ECHILD;
 	PROC *p;
@@ -460,10 +460,10 @@ static int searchExitState(PROC *proc,int pid,int gid,int state,int *stat_loc)
 				break;
 			}
 	}
-	else if(gid!=0)
+	else if(pgid!=0)
 	{
 		for(p=proc->child;p!=NULL;p=p->brother)
-			if(p->gid==gid)
+			if(p->pgid==pgid)
 			{
 				rest=0;
 				if(p->state&state)
@@ -515,6 +515,21 @@ void waitProc1()
 }
 
 
+/*
+ * GLOBAL
+ * Set/release wait terminal flag.
+ */
+void addNestCnt()
+{
+	get_current_task()->nest_count+=1;
+}
+
+void delNestCnt()
+{
+	get_current_task()->nest_count-=1;
+}
+
+
 /******************************************************************************************************
  *
  * < ץID롼ID >
@@ -524,10 +539,10 @@ void waitProc1()
 /*
  * PRIVATE
  * ץ롼פθ
- * parameters : search start process,gid
+ * parameters : search start process,pgid
  * return : process or Failed=NULL
  */
-static PROC *searchPg(PROC *proc,int gid)
+static PROC *searchPg(PROC *proc,pid_t pgid)
 {
 	PROC *rest;
 
@@ -536,9 +551,9 @@ static PROC *searchPg(PROC *proc,int gid)
 	{
 		if(proc->state!=TASK_EXIT)
 		{
-			if(proc->gid==gid)return proc;
+			if(proc->pgid==pgid)return proc;
 			if(proc->child!=NULL)
-				if((rest=searchPg(proc->child,gid))!=NULL)return rest;
+				if((rest=searchPg(proc->child,pgid))!=NULL)return rest;
 		}
 	}while((proc=proc->brother)!=NULL);
 
@@ -567,20 +582,20 @@ static inline int getPid(PROC *proc)
  * parameters : search start process or NULL=proc1,groupe ID
  * return : prosecc or Failed=NULL
  */
-PROC *getNextProc(PROC *proc,int gid)
+PROC *getNextProc(PROC *proc,pid_t pgid)
 {
 	/* proc1õ */
 	if(proc==NULL)proc=proc1;
 
 	/* μν֤Υץõ */
-	if(proc->child!=NULL)return searchPg(proc->child,gid);
+	if(proc->child!=NULL)return searchPg(proc->child,pgid);
 	else
 	{
 		do
 		{
-			if(proc->brother!=NULL)return searchPg(proc->brother,gid);
+			if(proc->brother!=NULL)return searchPg(proc->brother,pgid);
 			if((proc=proc->parent)==NULL)break;
-		}while(proc->gid==gid);
+		}while(proc->pgid==pgid);
 	}
 
 	return NULL;
@@ -612,216 +627,6 @@ PROC *getAllProc(PROC *proc)
 }
 
 
-/******************************************************************************************************
- *
- * < ȥ饹 >
- *
- *******************************************************************************************************/
-
-/*
- * PRIVATE
- * PROC_LINK¤Υɥ쥹PROCɥ쥹롣
- */
-static inline PROC *waitLinkToProc(PROC_LINK *p)
-{
-	return (PROC*)((uint)p-(uint)&((PROC*)0)->wait_next);
-}
-
-
-/*
- * GLOBAL
- * 󥰥륦ȡ
- * ա
- *  ƤӽФ˰wait->flag0Ƥɬפ롣
- * parameters : ߹¤,ॢ(ms),ȥե饰
- * return : 0,ॢ-1
- */
-int wait_intr(WAIT_INTR *wait,uint time,int flag)
-{
-	wait->proc=get_current_task();
-
-	cli();
-	{
-		/* ˳ߤ? */
-		if(wait->flag==1)
-		{
-			wait->flag=0;
-			sti();
-			return 0;
-		}
-		set_timer(time);
-
-		del_from_schedule(flag);
-		wait->flag=1;
-	}
-	sti();
-
-	wait_task();
-
-	return wait->flag-1;
-}
-
-void wake_intr(WAIT_INTR *wait,int flag)
-{
-	/* ץwait˳ߤ? */
-	if(wait->flag==0)
-	{
-		wait->flag=1;
-		return;
-	}
-
-	del_timer(wait->proc);
-	wait->flag=0;
-	/*
-	 * 塼ؤɲäϽκǸ˹Ԥʤߤ顢
-	 * ٤Τ̤Τޤwakeץ¹ԤƤޤ
-	 */
-	add_to_schedule(wait->proc,flag);
-/******************************************************
-{
-	PROC *p,*q;
-
-	for(p=q=get_current_task();;p=p->next)
-	{
-		printk("%x ",p);
-		if(p==p->next)
-		{
-			printk("%x\n",p);
-			break;
-		}
-		if(p->next==q)
-		{
-			printk("\n");
-			break;
-		}
-	}
-
-}
-******************************************************/
-}
-
-
-/*
- * GLOBAL
- * Ԥѥȡ
- * ա
- * PROC_LINKϺǽ鼫ʬؤΥ󥯤ꤵƤɬפ롣
- * paramerers : WAIT_PROC structure address.
- */
-void waitProc(PROC_LINK *wait)
-{
-	PROC_LINK *p=(PROC_LINK*)&get_current_task()->wait_next;
-
-
-	p->next=wait->next;
-	p->prev=wait;
-	wait->next->prev=p;
-	wait->next=p;
-
-	del_from_schedule(TASK_WAIT);
-	wait_task();
-}
-
-void wakeProc(PROC_LINK *wait)
-{
-	PROC *proc;
-	PROC_LINK *p;
-
-	while(wait->next==wait);	/* waitProc()ǤΥ󥯽Ԥ */
-
-	p=wait->next;
-	p->next->prev=wait;
-	wait->next=p->next;
-
-	proc=waitLinkToProc(p);
-	while(proc->state!=TASK_WAIT);
-	add_to_schedule(proc,TASK_WAIT);
-}
-
-
-/*
- * GLOBAL
- * ޥեꥦȡ
- * ñ
- * parameters : Wait queue
- */
-void waitSemaphore(WAIT_QUEUE *queue)
-{
-	PROC *proc;
-	PROC_LINK *p,*wait;
-
-
-	wait=(PROC_LINK*)&queue->wait_next;
-
-	enter_spinlock(&queue->gate);
-
-	if(queue->flag==0)
-	{
-		queue->flag=1;
-		wait->next=wait->prev=wait;
-		exit_spinlock(&queue->gate);
-
-		return;
-	}
-	else
-	{
-		proc=get_current_task();
-		p=(PROC_LINK*)&proc->wait_next;
-
-		/* Add process to wait queue */
-		p->next=wait;
-		p->prev=wait->prev;
-		wait->prev->next=p;
-		wait->prev=p;
-
-		exit_spinlock(&queue->gate);
-
-		del_from_schedule(TASK_WAIT);
-	}
-
-	wait_task();
-}
-
-void wakeSemaphore(WAIT_QUEUE *queue)
-{
-	PROC_LINK *p,*wait;
-
-
-	wait=(PROC_LINK*)&queue->wait_next;
-
-	/* Delete process from wait queue */
-	enter_spinlock(&queue->gate);
-	{
-		if(wait->next!=wait)
-		{
-			p=wait->next;
-			wait->next=p->next;
-			wait->next->prev=wait;
-
-			/* Add process to schedule */
-			add_to_schedule(waitLinkToProc(p),TASK_WAIT);
-		}
-		else queue->flag=0;
-	}
-	exit_spinlock(&queue->gate);
-}
-
-
-/*
- * GLOBAL
- * Set/release wait terminal flag.
- */
-void addNestCnt()
-{
-	get_current_task()->nest_count+=1;
-}
-
-void delNestCnt()
-{
-	get_current_task()->nest_count-=1;
-}
-
-
 /***************************************************************************************************
  *
  * System time
@@ -921,11 +726,13 @@ int sys_fork()
 	PROC *parent,*child;
 
 
-	parent=get_current_task();
-	if(parent->count_child>=MAX_CHILD)return -EAGAIN;
+	parent = get_current_task();
+	if (parent->count_child >= MAX_CHILD)
+		return -EAGAIN;
 
 	/* PROC¤ΤƤ */
-	if((child=(PROC*)kmalloc(sizeof(PROC)))==NULL)return -ENOMEM;
+	if ((child = kmalloc(sizeof(PROC))) == NULL)
+		return -ENOMEM;
 
 	/* 0 */
 	memset(&child->PROC_ZERO_TOP,0,(uint)&child->PROC_ZERO_END-(uint)&child->PROC_ZERO_TOP);
@@ -936,8 +743,9 @@ int sys_fork()
 
 	/* ID. */
 	child->pid=getPid(child);
-	child->gid=parent->gid;
+	child->pgid=parent->pgid;
 	child->uid=0;
+	child->gid=parent->gid;
 
 	/* å󥢥ɥ쥹(ü¤Υɥ쥹)򥳥ԡ */
 	child->ctlterm=parent->ctlterm;
@@ -952,10 +760,12 @@ int sys_fork()
 	if((child->signal_struct=setSignal(parent->signal_struct))==NULL)goto ERR;
 
 	/* ޡν */
-	if((child->timer_struct=setTimerStruct())==NULL)goto ERR;
+	setTimerStruct(child);
 
 	/* ҥץstack˥ƥȤꤹ */
-	if((esp=setContext((uint)stack))==0)return 0;		/* ҥץ */
+	esp = setContext((uint)stack);
+	if (esp == 0)
+		return 0;		/* ҥץ */
 
 	child->esp=esp;
 
@@ -977,7 +787,7 @@ ERR:
 
 int sys_exec(const char *path,char **argv,char **envp,volatile SYSCALL4_FRAME frame)
 {
-	enum{PARAM=sizeof(int)*4};	/* ƥॳΰο  Хȿ */
+	enum{PARAM=sizeof(frame.para)};	/* ƥॳΰΥ */
 
 	uint entry_addr,stack_esp;
 	int rest;
@@ -1011,7 +821,7 @@ int sys_exec(const char *path,char **argv,char **envp,volatile SYSCALL4_FRAME fr
 	frame.ds=USER_DATA_DES;
 	frame.eip=entry_addr;
 	frame.cs=USER_CODE_DES;
-	frame.esp=stack_esp-PARAM;
+	frame.user_esp=stack_esp-PARAM;
 	frame.ss=USER_DATA_DES;
 
 	return 0;
@@ -1055,7 +865,7 @@ void exit(int state,int signum)
 	releaseFileStruct(proc->file_struct);
 
 	/* ޡγ */
-	releaseTimer(proc->timer_struct);
+	releaseTimer(&proc->timer,&proc->alarm);
 
 	/* ʥγ */
 	releaseSignal(proc->signal_struct);
@@ -1103,7 +913,7 @@ int sys_wait(int pid,int *stat_loc,int options)
 
 	/* ǽλҥץ */
 	if(pid>0)rest=searchExitState(proc,pid,0,state,stat_loc);
-	else if(pid==0)rest=searchExitState(proc,0,proc->gid,state,stat_loc);
+	else if(pid==0)rest=searchExitState(proc,0,proc->pgid,state,stat_loc);
 	else if(pid==-1)rest=searchExitState(proc,0,0,state,stat_loc);
 	else rest=searchExitState(proc,0,-pid,state,stat_loc);
 
@@ -1116,7 +926,7 @@ int sys_wait(int pid,int *stat_loc,int options)
 
 	/* ҥץ */
 	if(pid>0)rest=searchExitState(proc,pid,0,state,stat_loc);
-	else if(pid==0)rest=searchExitState(proc,0,proc->gid,state,stat_loc);
+	else if(pid==0)rest=searchExitState(proc,0,proc->pgid,state,stat_loc);
 	else if(pid==-1)rest=searchExitState(proc,0,0,state,stat_loc);
 	else rest=searchExitState(proc,0,-pid,state,stat_loc);
 
@@ -1127,7 +937,7 @@ int sys_wait(int pid,int *stat_loc,int options)
 /*
  * Get process ID.
  */
-int sys_get_procid(int pid,PROC_ID *pd)
+int sys_get_procid(pid_t pid,PROC_ID *pd)
 {
 	PROC *proc;
 
@@ -1141,9 +951,10 @@ int sys_get_procid(int pid,PROC_ID *pd)
 	}
 
 	pd->pid=proc->pid;
-	pd->gid=proc->gid;
-	pd->uid=proc->uid;
+	pd->pgid=proc->pgid;
 	pd->ppid=proc->parent->pid;
+	pd->uid=proc->uid;
+	pd->gid=proc->gid;
 
 	return 0;
 }
@@ -1152,7 +963,7 @@ int sys_get_procid(int pid,PROC_ID *pd)
 /*
  * Set process ID.
  */
-int sys_set_procid(int pid,PROC_ID *pd)
+int sys_set_procid(pid_t pid,PROC_ID *pd)
 {
 	PROC *proc;
 
@@ -1164,20 +975,26 @@ int sys_set_procid(int pid,PROC_ID *pd)
 		if((proc=searchProcList(pid))==NULL)return -ESRCH;
 	}
 
-	if(pd->gid>=0)
+	if(pd->pgid>=0)
 	{
 		/* ץå꡼ */
 		if(isCtlproc(proc))return -EPERM;
 
 		/* ȥץλҥץǤʡ */
-		if(proc->gid!=get_current_task()->gid)return -EPERM;
+		if(proc->pgid!=get_current_task()->pgid)return -EPERM;
 
-		if(pd->gid==0)proc->gid=proc->pid;
-		else proc->gid=pd->gid;
+		if(pd->pgid==0)proc->pgid=proc->pid;
+		else proc->pgid=pd->pgid;
 	}
 	else if(pd->uid>=0)
 	{
-		proc->uid=pd->uid;
+		if(proc->uid==0)proc->uid=pd->uid;
+		else return -EPERM;
+	}
+	else if(pd->gid>=0)
+	{
+		if(proc->uid==0)proc->gid=pd->gid;
+		else return -EPERM;
 	}
 
 	return 0;
@@ -1262,7 +1079,9 @@ void set_tss(int des)
 int set_idle_proc(int cpu)
 {
 	/* Set process table */
-	if((idle_proc[cpu]=(PROC*)kmalloc(sizeof(PROC)))==NULL)return -ENOMEM;
+	idle_proc[cpu] = kmalloc(sizeof(PROC));
+	if (idle_proc[cpu] == NULL)
+		return -ENOMEM;
 	memset(idle_proc[cpu],0,sizeof(PROC));
 	idle_proc[cpu]->next=idle_proc[cpu]->prev=idle_proc[cpu];
 	idle_proc[cpu]->cpu=cpu;
