@@ -14,6 +14,7 @@
 
 #include<types.h>
 #include<segment.h>
+#include<time.h>
 #include<share/wait.h>
 
 
@@ -24,13 +25,13 @@ enum{
 	MAX_CHILD=8,		/* Max number of child processes */
 
 	/* ץξ֡ */
-	TASK_RUNNING=    1<<0,
-	TASK_WAIT=       1<<1,
-	TASK_DEVICE_WAIT=1<<2,	/* ǥХԤ */
-	TASK_SIGNAL_WAIT=1<<3,	/* ʥԤ */
-	TASK_TRACE=      1<<4,	/* ȥ졼档 */
-	TASK_EXIT=       1<<5,	/* Exit. */
-	TASK_CHILD_WAIT= 1<<6,	/* wait callԤ */
+	TASK_RUNNING=    1 << 0,
+	TASK_WAIT=       1 << 1,
+	TASK_DEVICE_WAIT=1 << 2,	/* ǥХԤ */
+	TASK_SIGNAL_WAIT=1 << 3,	/* ʥԤ */
+	TASK_TRACE=      1 << 4,	/* ȥ졼档 */
+	TASK_EXIT=       1 << 5,	/* Exit. */
+	TASK_CHILD_WAIT= 1 << 6,	/* wait callԤ */
 };
 
 
@@ -49,7 +50,8 @@ typedef struct PROC{
 	struct PROC *parent;		/* Parent process. */
 	struct PROC *brother;		/* Brother process */
 	struct PROC *child;			/* Child process */
-	ushort       nest_count;	/* ߥͥȥȡ */
+	uchar        wait_update;	/* ߥȡ */
+	uchar        nest_count;	/* ߥͥȥȡ */
 	ushort       count_child;	/* Number of child processes */
 	uint64       cpu_time[2];	/* CPU time,0:system 1:user. */
 	uint         ctime[2];		/* Children clock tick,0:system 1:user. */
@@ -57,11 +59,13 @@ typedef struct PROC{
 	uchar        signum;		/* Exit signal number. */
 	uchar        cpu;			/* running cpu number ƯƤʤ-1ʲ*/
 	uchar        state;			/* ץξ֡ */
-	int          pid;			/* process ID. */
-	int          gid;			/* groupe ID. */
-	int          uid;			/* user ID. */
+	pid_t        pid;			/* process ID. */
+	pid_t        pgid;			/* process groupe ID. */
+	uid_t        uid;			/* user ID. */
+	gid_t        gid;			/* user groupe ID. */
+	TIMER        timer;			/* ޡ¤Ρ */
+	TIMER        alarm;			/* 顼๽¤Ρ */
 	void        *ctlterm;		/* ߥʥ湽¤Ρ */
-	void        *timer_struct;	/* ޡ¤Ρ */
 	void        *signal_struct;	/* ʥ¤Ρ */
 	void        *mm_struct;		/* ꡼¤Υɥ쥹 */
 	void        *file_struct;	/* ե빽¤Υɥ쥹 */
@@ -74,26 +78,6 @@ typedef struct PROC_LIST{
 	int lockGate;
 }PROC_LIST;
 
-/* ץ󥯹¤ */
-typedef struct PROC_LINK{
-	struct PROC_LINK *next;
-	struct PROC_LINK *prev;
-}PROC_LINK;
-
-/* ץȥå塼¤ */
-typedef struct WAIT_QUEUE{
-	void *wait_next;			/* ȥ󥯤μΥץ */
-	void *wait_prev;			/* ȥ󥯤Υץ */
-	int flag;					/* Wait flag */
-	int gate;					/* Lock gate */
-}WAIT_QUEUE;
-
-/* ޡԤѹ¤ */
-typedef struct WAIT_INTR{
-	PROC *proc;					/* Ԥץ */
-	int flag;					/* ॢȥե饰 */
-}WAIT_INTR;
-
 /* cpuȤΥ¤ */
 typedef struct CPU{
 	int cpu;				/* number of cpu */
@@ -107,6 +91,9 @@ extern CPU cputask[];
 extern PROC_LIST procHash[];
 
 
+#define getProcAddr(address,member) (PROC*)((uint)address-(uint)&((PROC*)0)->member)
+
+
 /*
  * Get current cpu number
  * returns : current cpu number
@@ -142,12 +129,6 @@ extern void del_from_schedule(int);
 extern PROC *searchProcList(int);
 extern void waitProc1();
 extern int initProc1();
-extern void waitProc(PROC_LINK*);
-extern void wakeProc(PROC_LINK*);
-extern void waitSemaphore(WAIT_QUEUE*);
-extern void wakeSemaphore(WAIT_QUEUE*);
-extern int wait_intr(WAIT_INTR*,uint,int);
-extern void wake_intr(WAIT_INTR*,int);
 extern int init_cputask(int);
 extern int set_idle_proc(int);
 extern void set_tss(int);
