@@ -14,7 +14,7 @@
 #include<errno.h>
 #include<lib.h>
 #include<interrupt.h>
-#include<proc.h>
+#include<wait.h>
 #include<mp.h>
 #include<sp.h>
 #include<term.h>
@@ -94,16 +94,6 @@ static void set_boudrate(int,ushort);
 static int write_ttys(int,const char*,size_t);
 static int write_com(int,char*,size_t);
 static int detect_uart(int);
-static int write_ttys0(void*,size_t,size_t);
-static int write_ttys1(void*,size_t,size_t);
-static int ioctl_ttys0(int,int,...);
-static int ioctl_ttys1(int,int,...);
-
-
-static DEV_INFO com_info[2]={
-	{"ttys0",0,0,0,NULL,NULL,write_ttys0,ioctl_ttys0},
-	{"ttys1",0,0,0,NULL,NULL,write_ttys1,ioctl_ttys1}
-};
 
 
 /*
@@ -112,9 +102,6 @@ static DEV_INFO com_info[2]={
  */
 int com1_intr_handler()
 {
-/***************************************/
-	printk("Interrupt IRQ4\n");
-/***************************************/
 	switch(inb(reg[0].iir)&0xf)
 	{
 		case TRANSMIT_EMPTY:
@@ -137,9 +124,6 @@ int com1_intr_handler()
  */
 int com2_intr_handler()
 {
-/***************************************/
-	printk("Interrupt IRQ3\n");
-/***************************************/
 	switch(inb(reg[1].iir)&0xf)
 	{
 		case TRANSMIT_EMPTY:
@@ -149,7 +133,7 @@ int com2_intr_handler()
 		case RECEIVE_TIMEOUT:
 		case LINE_STATUS:
 		case MODEM_STATUS:
-		;
+			;
 	}
 
 	return 1;
@@ -305,6 +289,64 @@ int write_com(int com,char *buf,size_t count)
 }
 
 
+/************************************************************************************************
+ *
+ * System call interface
+ *
+ ************************************************************************************************/
+
+
+static int open0()
+{
+	return 0;
+}
+
+static int open1()
+{
+	return 0;
+}
+
+static int read0(void *buf,size_t count,size_t begin)
+{
+	return 0;
+}
+
+static int read1(void *buf,size_t count,size_t begin)
+{
+	return 0;
+}
+
+static int write_ttys0(void *buf,size_t count,size_t x)
+{
+	return write_ttys(0,(const char*)buf,count);
+}
+
+static int write_ttys1(void *buf,size_t count,size_t x)
+{
+	return write_ttys(1,(const char*)buf,count);
+}
+
+static int ioctl_ttys0(int cmd,...)
+{
+	return 0;
+}
+
+static int ioctl_ttys1(int cmd,...)
+{
+	return 0;
+}
+
+static int select0(int flag,int polling)
+{
+	return 1;
+}
+
+static int select1(int flag,int polling)
+{
+	return 1;
+}
+
+
 /************************************************************************************************
  *
  * Init COM
@@ -312,6 +354,12 @@ int write_com(int com,char *buf,size_t count)
  ************************************************************************************************/
 
 
+static DEV_INFO com_info[2]={
+	{"ttys0",0,0,0,open0,read0,write_ttys0,ioctl_ttys0,select0},
+	{"ttys1",0,0,0,open1,read1,write_ttys1,ioctl_ttys1,select1}
+};
+
+
 void init_com()
 {
 	int i;
@@ -383,36 +431,6 @@ void set_com_intr()
 		if(uart[i]>0)printk("ttys%d : %s\n",i,uart_name[uart[i]]);
 }
 
-
-/************************************************************************************************
- *
- * System call interface
- *
- ************************************************************************************************/
-
-
-/*
- * Write call
- */
-int write_ttys0(void *buf,size_t count,size_t x)
-{
-	return write_ttys(0,(const char*)buf,count);
-}
-
-int write_ttys1(void *buf,size_t count,size_t x)
-{
-	return write_ttys(1,(const char*)buf,count);
-}
-
-int ioctl_ttys0(int prm_num,int cmd,...)
-{
-	return 0;
-}
-
-int ioctl_ttys1(int prm_num,int cmd,...)
-{
-	return 0;
-}
 /***************************************************************************************/
 void test_com()
 {
