@@ -116,22 +116,9 @@ void start_tasking()
  *
  *************************************************************************************************/
 
-/*
- * 顼
- * parameters : error message
- */
-static void doError(const char *func,int errno)
-{
-	print_error(errno,func);
-	idle();
-}
-
 
 void main()
 {
-	int rest;
-
-
 	/* 㳰ϥɥꤹ롣 */
 	init_except();
 
@@ -150,7 +137,6 @@ void main()
 			write_tty=write_com2;
 			break;
 	}
-
 	/* TSSǥץɤ롣 */
 	set_tss(TSS_DES);
 
@@ -160,11 +146,17 @@ void main()
 	/* cpuեߥ꡼Υå */
 	cpu_family=(cpuid()>>8)&0xf;
 	if(cpu_family<CPU_FAMILY_PENTIUM)
-		doError("This cpu is not supported!",0);
+	{
+		PRINT_ERR("This cpu is not supported!");
+		idle();
+	}
 
 	/* ɥ쥹饤20ӥåܤͭˤ롣 */
 	if(enable_a20()==-1)
-		doError("A20 can't be enable. Boot stop!",0);
+	{
+		PRINT_ERR("A20 can't be enable. Boot stop!");
+		idle();
+	}
 
 	/* Apmν */
 	InitApm();
@@ -179,11 +171,14 @@ void main()
 	printk("cpu0 clock = %d HZ\n",count_cpu_clock());
 
 	/* ɥץ롣 */
-	if((rest=set_idle_proc(0))<0)
-		doError("set_idle_proc",-rest);
+	if (set_idle_proc(0) < 0)
+	{
+		PRINT_ERR("set_idle_proc error!");
+		idle();
+	}
 
 	/* ߤSMP롣 */
-	if(MFPS_addres)
+	if (MFPS_addres)
 	{
 		if(init_mp()==-1)idle();
 	}
@@ -195,7 +190,7 @@ void main()
 
 
 /* ǽΥ桼ץ¹ */
-extern int startExec(int,const char*,void*,void*);
+extern int startExec(const char*,void*,void*);
 
 
 void init()
@@ -205,7 +200,6 @@ void init()
 
 	/* ߳ϡ */
 	sti();
-
 	/* ǽΥץ process0 롣 */
     {
 		int error;
@@ -225,8 +219,11 @@ void init()
 	init_fs();
 
 	/* Init device system. */
-	if((rest=init_device())<0)
-		doError("init_device",-rest);
+	if (init_device() < 0)
+	{
+		PRINT_ERR("init_device error!");
+		idle();
+	}
 
 	/* Init PCI. */
 	initPci();
@@ -247,53 +244,76 @@ void init()
 
 	/* 󥽡ΥǥХϿ */
 	if((rest=registConsole())<0)
-		doError("registConsole",-rest);
+	{
+		PRINT_ERR("registConsole error!");
+		idle();
+	}
 
 	/* NULLǥХϿ */
 	if((rest=registNullDev())<0)
-		doError("registNullDev",-rest);
+	{
+		PRINT_ERR("registNullDev error!");
+		idle();
+	}
 
 	/* EXT2ե륷ƥν */
 	if((rest=initExt2tFs())<0)
-		doError("initExt2tFs",-rest);
+	{
+		PRINT_ERR("initExt2tFs error!");
+		idle();
+	}
 
 	/* ڡѵΰν */
 	if((rest=initBackupDev(PAGING_BACKUP_DEVICE))<0)
-		doError("initBackupDev",-rest);
+	{
+		PRINT_ERR("initBackupDev error!");
+		idle();
+	}
 	printk("Page backup device : %s\n",PAGING_BACKUP_DEVICE);
 
 	/* mount root filesystem. */
 	if((rest=mountRoot(ROOT_DEVICE,"ext2"))<0)
-		doError("mountRoot",-rest);
+	{
+		PRINT_ERR("mountRoot error!");
+		idle();
+	}
 	printk("Root device : %s\nRoot file system : ext2\n",ROOT_DEVICE);
 
 	/* Init keyboard driver. */
 	initKeyboard();
 
-/************** ͥåȥطν *******************************************/
-	/* Init network.  */
+	/* ͥåȥν  */
 	initNet();
 
-	/* Init ethernet driver. */
+	/* ͥåȥɥ饤Сν */
 	if((rest=initEther())<0)
-		doError("initEther",-rest);
-/***********************************************************************************/
+	{
+		PRINT_ERR("initEther error!");
+		idle();
+	}
+	
 	/* Init process0. */
 	if((rest=initProc1())<0)
-		doError("initProc0",-rest);
+	{
+		PRINT_ERR("initProc0 error!");
+		idle();
+	}
 
 	/* ǽΥ桼ץư롣 */
 	rest=sys_fork();
    	if(rest<0)
    	{
-   		doError("sys_fork",-rest);
+   		PRINT_ERR("Fork of first user process error!");
    		idle();
    	}
    	else if(rest==0)
    	{
    		/* 桼ץ¹Ԥ롣 */
-		if((rest=startExec(SYS_EXEC,"/shell",NULL,NULL))<0)
-			doError("startExec",-rest);
+		if((rest=startExec("/shell",NULL,NULL))<0)
+		{
+			PRINT_ERR("startExec error!");
+			idle();
+		}
 	}
 
 	/* ҥץexitԤ */
@@ -328,3 +348,9 @@ int sys_uname(struct utsname *buf)
 
 	return 0;
 }
+
+/****************************************************************************************************/
+void test_setup()
+{
+	printk("ok\n");
+}
