@@ -15,6 +15,7 @@
 #include<errno.h>
 #include<term.h>
 #include<signal.h>
+#include<test.h>
 
 
 /****************************************************************************************
@@ -99,24 +100,13 @@ static SIG_ACTION defaultAction[]={
  *
  *****************************************************************************************/
 
-/*
- * PRIVATE
- * ʥ빽¤Υɥ쥹顢ץɥ쥹롣
- * parameters : ʥ빽¤
- * return : ץ
- */
-static inline PROC *sigToProc(SIGNAL *sig_st)
-{
-	return (PROC*)((uint)sig_st-(uint)&((PROC*)0)->signal_struct);
-}
-
 
 /*
- * PRIVATE
+ * GLOBALk 
  * ʥ¹ԡ
  * parameters : user esp address
  */
-static inline void doSignal(uint *user_esp)
+void doSignal(uint *user_esp)
 {
 	SIGNAL *sig_st;
 	PROC *proc=get_current_task();
@@ -124,7 +114,7 @@ static inline void doSignal(uint *user_esp)
 
 
 	if(proc->nest_count>0)return;
-	
+
 	sig_st=proc->signal_struct;
 
 	/* ʥϥɥμ¹ԡ */
@@ -157,28 +147,6 @@ static inline void doSignal(uint *user_esp)
 }
 
 
-/*
- * PRIVATE
- * ̾ߤΥʥ¹ԡ
- * parameters : user esp
- */
-void doSignalFromTrap(TRAP_FRAME frame)
-{
-	doSignal(&frame.user_esp);
-}
-
-
-/*
- * PRIVATE
- * 㳰ߤΥʥ¹ԡ
- * parameters : user esp
- */
-void doSignalFromExcept(EXCEPT_FRAME frame)
-{
-	doSignal(&frame.user_esp);
-}
-
-
 /*
  * GLOBAL
  * ʥ
@@ -192,7 +160,7 @@ int sendSignal(PROC *proc,int signal)
 	int sig;
 	SIGNAL *sig_st;
 
-
+	
 	sig_st=proc->signal_struct;
 
 	/* ʥ뤬̵ˤƤʤ */
@@ -203,10 +171,16 @@ int sendSignal(PROC *proc,int signal)
 
 	if(sig!=0)
 	{
-		/* ʥԤʤ鵯 */
-		add_to_schedule(proc,TASK_SIGNAL_WAIT);
-
-		sig_st->signal|=sig;
+		enter_spinlock(&sig_st->lock);
+		{
+			sig_st->signal|=sig;
+			if(proc->state==TASK_SIGNAL_WAIT)
+			{
+				add_to_schedule(proc,TASK_SIGNAL_WAIT);
+				sig_st->signal&=~(1<<SIGCONT);					/* SIGCONTϥꥢ */
+			}
+		}
+		exit_spinlock(&sig_st->lock);
 	}
 
 	return 0;
@@ -221,20 +195,53 @@ int sendSignal(PROC *proc,int signal)
  */
 int forceSendSignal(PROC *proc,int signal)
 {
-	SIGNAL *sig_st;
+	SIGNAL *sig = proc->signal_struct;
 
 
-	sig_st=proc->signal_struct;
-
-	/* ʥԤʤ鵯 */
-	add_to_schedule(sigToProc(sig_st),TASK_SIGNAL_WAIT);
-
-	sig_st->signal|=1<<signal;
+	enter_spinlock(&sig->lock);
+	{
+		sig->signal |= 1 << signal;
+		if(proc->state == TASK_SIGNAL_WAIT)
+		{
+			add_to_schedule(proc,TASK_SIGNAL_WAIT);
+			sig->signal &= ~(1 << SIGCONT);			/* SIGCONTϥꥢ */
+		}
+	}
+	exit_spinlock(&sig->lock);
 
 	return 0;
 }
 
 
+/*
+ * GLOBAL
+ * ʥԤ
+ */
+void sigWait()
+{
+	SIGNAL *sig=get_current_task()->signal_struct;
+
+
+	enter_spinlock(&sig->lock);
+	{
+		if(sig->signal!=0)
+		{
+			exit_spinlock(&sig->lock);
+			
+			sig->signal&=~(1<<SIGCONT);					/* SIGCONTϥꥢ */
+			return ;
+		}
+
+		cli();
+		del_from_schedule(TASK_SIGNAL_WAIT);
+	}
+	exit_spinlock(&sig->lock);
+	sti();
+
+	wait_task();
+}
+
+
 /*
  * GLOBAL
  * fork()Υʥ뵡ꡣ
@@ -248,10 +255,9 @@ SIGNAL *setSignal(SIGNAL *parent_sig)
 
 	if((sig_st=kmalloc(sizeof(SIGNAL)))==NULL)return NULL;
 
-	memcpy(sig_st->action,parent_sig->action,sizeof(SIGNAL));
+	memcpy(sig_st,parent_sig,sizeof(SIGNAL));
 	sig_st->signal=0;
 	sig_st->reserve=0;
-	sig_st->mask=parent_sig->mask;
 
 	return sig_st;
 }
@@ -276,7 +282,7 @@ void releaseSignal(SIGNAL *sig_st)
 void resetSignal(SIGNAL *sig_st)
 {
 	/* ǥեȥ򥳥ԡ */
-	memcpy(sig_st->action,defaultAction,SIG_ACTIONS*sizeof(SIG_ACTION));
+	memcpy(sig_st->action,defaultAction,sizeof(SIG_ACTION)*SIG_ACTIONS);
 
 	memset(sig_st->actMask,0,sizeof(uint)*SIG_ACTIONS);
 	sig_st->signal=0;
@@ -329,8 +335,9 @@ SIGNAL *initSignal()
 	if((sig_st=kmalloc(sizeof(SIGNAL)))==NULL)return NULL;
 
 	/*  */
-	resetSignal(sig_st);
+	resetSignal(sig_st);	
 	sig_st->mask=0;
+	sig_st->lock=0;
 
 	return sig_st;
 }
@@ -362,7 +369,6 @@ int sys_sigaction(int signum,const struct sigaction *act,struct sigaction *oldac
 
 		oldact->sa_mask=signal->actMask[signum];
 	}
-
 	if(act!=NULL)
 	{
 		if((signum==SIGKILL)||(signum==SIGSTOP))return -EINVAL;		/* SIGKILLSIGSTOPѹԲġ */
@@ -409,9 +415,9 @@ int sys_sigprocmask(int how,uint *set,uint *oldset)
 }
 
 
-int sys_kill(int pid,int sig)
+int sys_kill(pid_t pid,int sig)
 {
-	int gid;
+	pid_t pgid;
 	int rest;
 	PROC *proc;
 
@@ -427,8 +433,8 @@ int sys_kill(int pid,int sig)
 	{
 		rest=-ESRCH;
 		proc=get_current_task();
-		gid=proc->gid;
-		while((proc=getNextProc(proc,gid))!=NULL)
+		pgid=proc->pgid;
+		while((proc=getNextProc(proc,pgid))!=NULL)
 			if((rest=sendSignal(proc,sig))<0)break;
 	}
 	else if(pid==-1)	/* ץ */
@@ -440,8 +446,8 @@ int sys_kill(int pid,int sig)
 	else				/* pid<-1 -pidΥ롼ס */
 	{
 		rest=-ESRCH;
-		gid=-pid;
-		for(proc=NULL;(proc=getNextProc(proc,gid))!=NULL;)
+		pgid=-pid;
+		for(proc=NULL;(proc=getNextProc(proc,pgid))!=NULL;)
 			if((rest=sendSignal(proc,sig))<0)break;
 	}
 
