@@ -11,7 +11,7 @@
 
 
 #include<proc.h>
-#include<share/signal.h>
+#include<sys/signal.h>
 
 
 #ifndef ASM_FILE
@@ -26,11 +26,14 @@ typedef struct{
 	uint mask;						/* signal mask. */
 	uint beforeEsp;					/* 桼ϥɥܹΥͥesp */
 	uint beforeStackEntry;			/* 桼ϥɥܹΥͥ륹åȥ꡼ */
+	int lock;						/* ʥԤƱå */
 }SIGNAL;
 
 
+extern void doSignal(uint *user_esp);
 extern int sendSignal(PROC*,int);
 extern int forceSendSignal(PROC*,int);
+extern void sigWait();
 extern SIGNAL *setSignal(SIGNAL*);
 extern void releaseSignal(SIGNAL*);
 extern void resetSignal(SIGNAL*);
