@@ -27,7 +27,7 @@
 int isForegrouond(PROC *proc)
 {
 	if(proc->ctlterm==NULL)return 0;
-	return ((TERM_CTL*)proc->ctlterm)->foregrnd==proc->gid;
+	return ((TERM_CTL*)proc->ctlterm)->foregrnd==proc->pgid;
 }
 
 
