@@ -9,24 +9,29 @@
 #define test_h
 
 
+#include<types.h>
+#include<proc.h>
+
+
 #ifndef ASM_FILE
 
 
 extern void test_hd();
 extern void test_fd();
 extern void test_com();
-extern void test_time();
+extern void test_time(uint);
 extern void test_device();
-extern void test_fs();
+extern void test_fs(PROC*);
 extern void test_orig_fs();
 extern void test_fatFs();
 extern void test_elf(void*);
-extern int test_ext2();
-extern int test_mm();
+extern int test_ext2(int);
+extern int test_mm(uint*);
 extern void test_proc();
 extern void test_pci();
 extern int test_arp();
 extern void test_tcp();
+extern void test_mp();
 
 
 #endif
