@@ -12,7 +12,7 @@
 #define types_h
 
 
-#include<share/types.h>
+#include<sys/types.h>
 #include<share/stdint.h>
 
 
