@@ -8,7 +8,7 @@
 
 
 #include<share/stdint.h>
-#include<share/socket.h>
+#include<sys/socket.h>
 
 
 #ifndef ASM_FILE
