@@ -25,7 +25,14 @@
 #define	TSS_DES         0x50				/* TSSǥץ͡ */
 #define GATE_DES        (TSS_DES+MAX_CPU*8)	/* value of Call gate discriptor. */
 
-#define GATE_SEGMENT GATE_DES
+/* ƥॳ륲ȥ */
+#define SYSCALL0_SEG GATE_DES
+#define SYSCALL1_SEG GATE_DES+8
+#define SYSCALL2_SEG GATE_DES+16
+#define SYSCALL3_SEG GATE_DES+24
+#define SYSCALL4_SEG GATE_DES+32
+#define SYSCALL5_SEG GATE_DES+40
+#define SYSCALL6_SEG GATE_DES+48
 
 
 #endif
