@@ -7,8 +7,9 @@
 #define share_stat_h
 
 
-#include<share/types.h>
+#include<sys/types.h>
 #include<share/stdint.h>
+#include<share/time.h>
 
 
 #ifndef ASM_FILE
@@ -28,6 +29,13 @@ struct stat{
 	time_t    st_ctime;		/* Time of last status change. */
 	size_t    st_crtpos;	/* option,current file position. */
 	uint32_t  st_blksize;	/* optimal blocksize for I/O */
+	dev_t	  st_rdev;		/* device type */
+	uint32_t  st_flags;		/* user defined flags for file */
+	int64_t	  st_blocks;	/* blocks allocated for file */
+	/* POSIX4. */
+	struct	timespec st_atimespec;	/* time of last access */
+	struct	timespec st_mtimespec;	/* time of last data modification */
+	struct	timespec st_ctimespec;	/* time of last file status change */
 };
 
 
