@@ -10,6 +10,9 @@
 #ifndef ASM_FILE
 
 
+#include<sys/types.h>
+
+
 enum{
 	/* Symbolic constants for the lseek() and fcntl() function. */
 	SEEK_CUR=1,
@@ -29,10 +32,11 @@ enum{
 
 
 typedef struct{
-	int pid;
-	int gid;
-	int uid;
-	int ppid;
+	pid_t pid;
+	pid_t pgid;
+	pid_t ppid;
+	uid_t uid;
+	gid_t gid;
 }PROC_ID;
 
 
