@@ -10,7 +10,7 @@
 #ifndef ASM_FILE
 
 
-#include<share/types.h>
+#include<sys/types.h>
 
 
 struct utimbuf{
