@@ -7,7 +7,7 @@
 #define termios_h
 
 
-#include<share/termios.h>
+#include<sys/termios.h>
 
 
 extern int     tcdrain(int);
