@@ -71,6 +71,7 @@ enum{
 	_PC_REC_XFER_ALIGN,
 	_PC_SYNC_IO,
 	_PC_VDISABLE,
+	_POSIX_VDISABLE=-1,
 
 	/* Symbolic constants for the sysconf() function. */
 	_SC_2_C_BIND=0,
@@ -190,12 +191,17 @@ extern int          execlp(const char *, const char *, ...);
 extern int          execv(const char *, char *const []);
 extern int          execvp(const char *, char *const []);
 extern void        _exit(int);
+extern int          fchdir(int);
+extern int          fchown(int, uid_t, gid_t);
 extern pid_t        fork(void);
 extern long         fpathconf(int, int);
+extern int          fsync(int);
+extern int          ftruncate(int, off_t);
 extern char        *getcwd(char *, size_t);
 extern gid_t        getegid(void);
 extern uid_t        geteuid(void);
 extern gid_t        getgid(void);
+extern int          gethostname(char *, size_t);
 extern char        *getlogin(void);
 extern pid_t        getpgid(pid_t);
 extern pid_t        getpid(void);
@@ -204,23 +210,28 @@ extern pid_t        getpgrp(void);
 extern pid_t        getppid(void);
 extern uid_t        getuid(void);
 extern int          isatty(int);
+extern int          lchown(const char *, uid_t, gid_t);
 extern int          link(const char *, const char *);
 extern off_t        lseek(int, off_t, int);
 extern long         pathconf(const char *, int);
 extern int          pause(void);
 extern int          pipe(int [2]);
 extern ssize_t      read(int, void *, size_t);
+extern ssize_t      readlink(const char *, char *, size_t);
 extern int          rmdir(const char *);
 extern int          setegid(gid_t);
 extern int          seteuid(uid_t);
 extern int          setgid(gid_t);
+extern int          sethostname(const char*, size_t);
 extern int          setpgid(pid_t, pid_t);
 extern pid_t        setsid(void);
 extern int          setuid(uid_t);
 extern unsigned     sleep(unsigned);
+extern int          symlink(const char *, const char *);
 extern long         sysconf(int);
 extern pid_t        tcgetpgrp(int);
 extern int          tcsetpgrp(int, pid_t);
+extern int          truncate(const char *, off_t);
 extern char        *ttyname(int);
 extern ssize_t      write(int, const void *, size_t);
 extern int          unlink(const char *);
@@ -228,16 +239,18 @@ extern int          getopt(int argc, char * const argv[], const char *optstring)
 
 
 /* ץޥ */
-#define ROUNDUP(a,b) (((uint)(a)+(b)-1)/(b)*(b))
-#define ROUNDDOWN(a,b) ((uint)(a)/(b)*(b))
-#define	MIN(a,b) (((a)<(b))?(a):(b))
-#define	MAX(a,b) (((a)>(b))?(a):(b))
+#define ROUNDUP(a,b)	(((uint)(a)+(b)-1)/(b)*(b))
+#define ROUNDDOWN(a,b)	((uint)(a)/(b)*(b))
+#define	howmany(x, y)	(((x)+((y)-1))/(y))
+#define	MIN(a,b)		(((a)<(b))?(a):(b))
+#define	MAX(a,b)		(((a)>(b))?(a):(b))
 
 /* ץؿ */
 extern int brk(void*);
 extern int reboot(int);
 extern void *sbrk(ptrdiff_t);
 extern void sync();
+extern int  mkstemp (char *);
 
 
 #endif
